@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2020 Guido Dhondt
+!     Copyright (C) 1998-2021 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
      &     ntie_,nbody_,nprop_,ipoinpc,nevdamp_,npt_,nslavs,nkon_,mcs,
      &     mortar,ifacecount,nintpoint,infree,nheading_,nobject_,
      &     iuel,iprestr,nstam,ndamp,nef,nbounold,nforcold,nloadold,
-     &     nbodyold,mpcend,irobustdesign)
+     &     nbodyold,mpcend,irobustdesign,nfc_,ndc_)
 !     
 !     calculates a conservative estimate of the size of the 
 !     fields to be allocated
@@ -46,8 +46,8 @@
       character*5 llab
       character*8 label
       character*20 mpclabel
-      character*81 set(*),noset,elset,slavset,mastset,noelset,
-     &     surface,slavsets,slavsett,mastsets,mastsett
+      character*81 set(*),noset,elset,slavset,mastset,noelset,submset,
+     &     surface,slavsets,slavsett,mastsets,mastsett,surfset
       character*132 jobnamec(*),textpart(16)
 !     
       integer nload_,nforc_,nboun_,nk_,ne_,nmpc_,nset_,nalset_,
@@ -60,12 +60,13 @@
      &     neb32,nn,nflow,nradiate,irestartread,irestartstep,icntrl,
      &     irstrt(*),ithermal(*),nener,nstate_,ipoinp(2,*),inp(3,*),
      &     ntie_,nbody_,nprop_,ipoinpc(0:*),nevdamp_,npt_,nentries,
-     &     iposs,iposm,nslavs,nlayer,nkon_,nopeexp,k,iremove,mcs,
+     &     iposs,iposm,nslavs,nlayer,nkon_,nopeexp,iremove,mcs,
      &     ifacecount,nintpoint,mortar,infree(4),nheading_,icfd,
      &     multslav,multmast,nobject_,numnodes,iorientation,id,
      &     irotation,itranslation,nuel,iuel(4,*),number,four,
      &     iprestr,nstam,ier,ndamp,nef,nbounold,nforcold,nloadold,
-     &     nbodyold,mpcend,irobustdesign(3)
+     &     nbodyold,mpcend,irobustdesign(3),iflag,network,
+     &     nsubmodel,nfc_,ndc_
 !     
       real*8 temperature,tempact,xfreq,tpinc,tpmin,tpmax
 !     
@@ -75,6 +76,10 @@
 !     =0: pure mechanical analysis
 !     =1: pure CFD analysis
 !     =2: mixed mechanical/cfd analysis
+!     
+!     mi(1): # of integration points
+!     mi(2): # of dofs per node
+!     mi(3): # of layers in the elements
 !     
       icfd=-1
 !     
@@ -97,7 +102,8 @@
       enddo
 !     
       istat=0
-!     
+!
+      nsubmodel=0
       nset_=0
       maxrmeminset=0
       necper=0
@@ -204,23 +210,26 @@
 !     check for element set
 !     
             elset(ipos:ipos)='E'
-            do i=1,nset_
-              if(set(i).eq.elset) then
+            call cident81(set,elset,nset_,id)
+            i=nset_+1
+            if(id.gt.0) then
+              if(set(id).eq.elset) then
+                i=id
                 nboun_=nboun_+ibound*meminset(i)
                 if(ntrans_.gt.0)then
                   nmpc_=nmpc_+ibound*meminset(i)
                   memmpc_=memmpc_+4*ibound*meminset(i)
                   nk_=nk_+meminset(i)
                 endif
-                exit
               endif
-            enddo
+            endif
             if(i.gt.nset_) then
 !     
 !     check for facial surface
 !     
               elset(ipos:ipos)='T'
-              do i=1,nset_
+              call cident81(set,elset,nset_,i)
+              if(i.gt.0) then
                 if(set(i).eq.elset) then
                   nboun_=nboun_+ibound*meminset(i)
                   if(ntrans_.gt.0)then
@@ -228,9 +237,8 @@
                     memmpc_=memmpc_+4*ibound*meminset(i)
                     nk_=nk_+meminset(i)
                   endif
-                  exit
                 endif
-              enddo
+              endif
             endif
           endif
         enddo
@@ -275,7 +283,8 @@
             noset(81:81)=' '
             ipos=index(noset,' ')
             noset(ipos:ipos)='N'
-            do i=1,nset_
+            call cident81(set,noset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.noset) then
                 nboun_=nboun_+ibound*meminset(i)
                 if(ntrans_.gt.0)then
@@ -283,11 +292,21 @@
                   memmpc_=memmpc_+4*ibound*meminset(i)
                   nk_=nk_+meminset(i)
                 endif
-                exit
               endif
-            enddo
+            endif
           endif
         enddo
+      elseif(textpart(1)(1:4).eq.'*CFD') then
+        iflag=1
+        do i=2,n
+          if(textpart(i)(1:10).eq.'TURBULENCE') then
+            iflag=iflag+1
+          endif
+        enddo
+        if(iflag.eq.2) mi(2)=max(mi(2),6)
+        write(*,*) 'mi(2)= ',mi(2)
+        call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &       ipoinp,inp,ipoinpc)
       elseif(textpart(1)(1:6).eq.'*CFLUX') then
         nam_=nam_+1
         namtot_=namtot_+1
@@ -304,12 +323,12 @@
             noset(81:81)=' '
             ipos=index(noset,' ')
             noset(ipos:ipos)='N'
-            do i=1,nset_
+            call cident81(set,noset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.noset) then
                 nforc_=nforc_+meminset(i)
-                exit
               endif
-            enddo
+            endif
           endif
         enddo
       elseif(textpart(1)(1:6).eq.'*CLOAD') then
@@ -332,16 +351,16 @@
             noset(81:81)=' '
             ipos=index(noset,' ')
             noset(ipos:ipos)='N'
-            do i=1,nset_
+            call cident81(set,noset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.noset) then
                 if(ntrans_.eq.0) then
                   nforc_=nforc_+meminset(i)
                 else
                   nforc_=nforc_+3*meminset(i)
                 endif
-                exit
               endif
-            enddo
+            endif
           endif
         enddo
       elseif((textpart(1)(1:13).eq.'*CONDUCTIVITY').or.
@@ -397,18 +416,20 @@
           endif
         enddo
         if(surface(1:1).ne.' ') then
-          do i=1,nset_
-            surface(ipos:ipos)='T'
+          surface(ipos:ipos)='T'
+          call cident81(set,surface,nset_,i)
+          if(i.gt.0) then
             if(set(i).eq.surface) then
               numnodes=8*meminset(i)
-              exit
             endif
-            surface(ipos:ipos)='S'
+          endif
+          surface(ipos:ipos)='S'
+          call cident81(set,surface,nset_,i)
+          if(i.gt.0) then
             if(set(i).eq.surface) then
               numnodes=meminset(i)
-              exit
             endif
-          enddo
+          endif
         endif
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &       ipoinp,inp,ipoinpc)
@@ -462,8 +483,11 @@
 !     and static pressure
 !     
         nk_=nk_+1
-        nmpc_=nmpc_+5*ncs_
-        memmpc_=memmpc_+125*ncs_
+!     
+!     change on 11th of Dec. 2020
+!     
+        nmpc_=nmpc_+(mi(2)+1)*ncs_
+        memmpc_=memmpc_+25*(mi(2)+1)*ncs_
         ntrans_=ntrans_+1
         do
           call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
@@ -540,6 +564,13 @@
       elseif(textpart(1)(1:22).eq.'*DEFORMATIONPLASTICITY') then
         ncmat_=max(5,ncmat_)
         ntmatl=0
+!
+!     the deformation plasticity model does not have internal
+!     variables. Nevertheless, space is reserved here for an
+!     equivalent plastic strain (used in sensitivity procedures
+!     to minimize plasticity)        
+!
+        nstate_=max(1,nstate_)
         do
           call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &         ipoinp,inp,ipoinpc)
@@ -560,6 +591,13 @@
         ntie_=ntie_+1   
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &       ipoinp,inp,ipoinpc)        
+      elseif(textpart(1)(1:15).eq.'*DESIGNRESPONSE') then
+        do
+          call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &         ipoinp,inp,ipoinpc)
+          if((istat.lt.0).or.(key.eq.1)) exit
+          nobject_=nobject_+1
+        enddo    
       elseif(textpart(1)(1:21).eq.'*DISTRIBUTINGCOUPLING') then
         nmpc_=nmpc_+3
         memmpc_=memmpc_+3
@@ -576,12 +614,12 @@
             noset(81:81)=' '
             ipos=index(noset,' ')
             noset(ipos:ipos)='N'
-            do i=1,nset_
+            call cident81(set,noset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.noset) then
                 memmpc_=memmpc_+3*meminset(i)
-                exit
               endif
-            enddo
+            endif
           endif
         enddo
       elseif(textpart(1)(2:13).eq.'DISTRIBUTING') then
@@ -620,29 +658,16 @@
 !     
             npt_=max(npt_,numnodes)
 !     
-            nmpc_=nmpc_+3*npt_+3
-            memmpc_=memmpc_+6*npt_+3*(npt_+1)
-            nk_=nk_+npt_
+            nfc_=nfc_+3*numnodes
+            nforc_=nforc_+3*numnodes
+            ndc_=ndc_+3
             itranslation=1
           endif
 !     
 !     rotational dofs
 !     
           if(ibound.gt.0) then
-            if(irotation.eq.0) then
-!     
-!     a MPC connecting the dofs 4-6 to dofs 1-3 of
-!     a rotational node; generation of a inhomogeneous
-!     node
-!     
-              nmpc_=nmpc_+3
-              memmpc_=memmpc_+6
-              nk_=nk_+4
-              irotation=1
-            endif
-            nmpc_=nmpc_+ibound
-            memmpc_=memmpc_+ibound*(3*npt_+2)
-            nboun_=nboun_+ibound
+            ndc_=ndc_+ibound
           endif
         enddo
 !     
@@ -697,31 +722,33 @@
 !     check for element set
 !     
             elset(ipos:ipos)='E'
-            do i=1,nset_
-              if(set(i).eq.elset) then
+            call cident81(set,elset,nset_,id)
+            i=nset_+1
+            if(id.gt.0) then
+              if(set(id).eq.elset) then
+                i=id
                 nload_=nload_+meminset(i)
                 if(massflow) then
                   nmpc_=nmpc_+meminset(i)
                   memmpc_=memmpc_+3*meminset(i)
                 endif
-                exit
               endif
-            enddo
+            endif
             if(i.gt.nset_) then
 !     
 !     check for facial surface
 !     
               elset(ipos:ipos)='T'
-              do i=1,nset_
+              call cident81(set,elset,nset_,i)
+              if(i.gt.0) then
                 if(set(i).eq.elset) then
                   nload_=nload_+meminset(i)
                   if(massflow) then
                     nmpc_=nmpc_+meminset(i)
                     memmpc_=memmpc_+3*meminset(i)
                   endif
-                  exit
                 endif
-              enddo
+              endif
             endif
           endif
         enddo
@@ -814,12 +841,22 @@
           ipos=index(elset,' ')
           elset(ipos:ipos)='E'
           ielset=1
-          do js=1,nset_
-            if(set(js).eq.elset) exit
-          enddo
+          call cident81(set,elset,nset_,id)
+          js=nset_+1
+          if(id.gt.0) then
+            if(set(id).eq.elset) js=id
+          endif
           if(js.gt.nset_) then
             nset_=nset_+1
-            set(nset_)=elset
+            do j=nset_,id+2,-1
+              meminset(j)=meminset(j-1)
+              rmeminset(j)=rmeminset(j-1)
+              set(j)=set(j-1)
+            enddo
+            js=id+1
+            set(js)=elset
+            meminset(js)=0
+            rmeminset(js)=0
           endif
         elseif(textpart(i)(1:5).eq.'TYPE=') then
           read(textpart(i)(6:13),'(a8)') label
@@ -1022,6 +1059,8 @@ c     Bernhardi end
               write(*,*) '       ',label
               call inputerror(inpc,ipoinpc,iline,
      &             "*ELEMENT%",ier)
+              call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &             ipoinp,inp,ipoinpc)
               cycle loop
             endif
           endif
@@ -1119,15 +1158,22 @@ c     !
 !     
 !     check whether new set name or old one
 !     
-        do js=1,nset_
-          if(set(js).eq.noelset) exit
-        enddo
-        if(js.gt.nset_) then
+        call cident81(set,noelset,nset_,id)
+        nn=nset_+1
+        if(id.gt.0) then
+          if(set(id).eq.noelset) nn=id
+        endif
+        if(nn.gt.nset_) then
           nset_=nset_+1
-          set(nset_)=noelset
-          nn=nset_
-        else
-          nn=js
+          do j=nset_,id+2,-1
+            meminset(j)=meminset(j-1)
+            rmeminset(j)=rmeminset(j-1)
+            set(j)=set(j-1)
+          enddo
+          nn=id+1
+          set(nn)=noelset
+          meminset(nn)=0
+          rmeminset(nn)=0
         endif
 !     
         if((n.gt.2).and.(textpart(3)(1:8).eq.'GENERATE')) then
@@ -1169,15 +1215,15 @@ c     !
                 else
                   noelset(ipos:ipos)='E'
                 endif
-                do j=1,nset_
-                  if(noelset.eq.set(j)) then
+                call cident81(set,noelset,nset_,j)
+                if(j.gt.0) then
+                  if(set(j).eq.noelset) then
                     meminset(nn)=meminset(nn)+
      &                   meminset(j)
                     rmeminset(nn)=rmeminset(nn)+
      &                   rmeminset(j)
-                    exit
                   endif
-                enddo
+                endif
               else
                 meminset(nn)=meminset(nn)+1
                 rmeminset(nn)=rmeminset(nn)+1
@@ -1282,12 +1328,21 @@ c     !
         ncmat_=max(11,ncmat_)
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &       ipoinp,inp,ipoinpc)
+      elseif(textpart(1)(1:20).eq.'*GEOMETRICCONSTRAINT') then
+        do   
+          call getnewline(inpc,textpart,istat,n,key,iline,ipol,
+     &      inl,ipoinp,inp,ipoinpc)
+          if((istat.lt.0).or.(key.eq.1)) exit
+          nobject_ = nobject_ + 1
+        enddo
       elseif(textpart(1)(1:8).eq.'*HEADING') then
         if(nheading_.ne.0) then
           write(*,*) '*ERROR in allocation: more than 1'
           write(*,*) '       *HEADING card in the input deck'
           call inputerror(inpc,ipoinpc,iline,
      &         "*HEADING%",ier)
+          call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &         ipoinp,inp,ipoinpc)
           cycle loop
         endif
         do
@@ -1546,8 +1601,9 @@ c     !
               noelset(81:81)=' '
               ipos=index(noelset,' ')
               noelset(ipos:ipos)='N'
-              do j=1,nset_
-                if(noelset.eq.set(j)) then
+              call cident81(set,noelset,nset_,j)
+              if(j.gt.0) then
+                if(set(j).eq.noelset) then
                   if(mpclabel(1:8).eq.'STRAIGHT') then
                     nk_=nk_+2*meminset(j)
                     nmpc_=nmpc_+2*meminset(j)
@@ -1563,9 +1619,8 @@ c     !
                   else
                     memmpc_=memmpc_+meminset(j)
                   endif
-                  exit
                 endif
-              enddo
+              endif
             else
               if(mpclabel(1:8).eq.'STRAIGHT') then
                 nk_=nk_+2
@@ -1619,12 +1674,22 @@ c     !
           ipos=index(noset,' ')
           noset(ipos:ipos)='N'
           inoset=1
-          do js=1,nset_
-            if(set(js).eq.noset) exit
-          enddo
+          call cident81(set,noset,nset_,id)
+          js=nset_+1
+          if(id.gt.0) then
+            if(set(id).eq.noset) js=id
+          endif
           if(js.gt.nset_) then
             nset_=nset_+1
-            set(nset_)=noset
+            do j=nset_,id+2,-1
+              meminset(j)=meminset(j-1)
+              rmeminset(j)=rmeminset(j-1)
+              set(j)=set(j-1)
+            enddo
+            js=id+1
+            set(js)=noset
+            meminset(js)=0
+            rmeminset(js)=0
           endif
         endif
       enddo loop3
@@ -1654,12 +1719,9 @@ c     !
           nprint_=nprint_+n
         enddo
       elseif(textpart(1)(1:10).eq.'*OBJECTIVE') then
-        do
-          call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
-     &         ipoinp,inp,ipoinpc)
-          if((istat.lt.0).or.(key.eq.1)) exit
-          nobject_=nobject_+1
-        enddo    
+        nobject_=nobject_+1
+        call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &       ipoinp,inp,ipoinpc) 
       elseif(textpart(1)(1:12).eq.'*ORIENTATION') then
         norien_=norien_+1
         do
@@ -1710,7 +1772,8 @@ c     !
           endif
         enddo
         if(surface(1:1).ne.' ') then
-          do i=1,nset_
+          call cident81(set,surface,nset_,i)
+          if(i.gt.0) then
             if(set(i).eq.surface) then
 !     
 !     worst case: 8 nodes per element face
@@ -1737,10 +1800,9 @@ c     !
      &             +16*meminset(i)
      &             +48*meminset(i)+1
      &             +12*(8*meminset(i)-1)
-              exit
 !     
             endif
-          enddo
+          endif
         endif
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &       ipoinp,inp,ipoinpc)
@@ -1762,13 +1824,13 @@ c     !
             elset(81:81)=' '
             ipos=index(elset,' ')
             elset(ipos:ipos)='E'
-            do i=1,nset_
+            call cident81(set,elset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.elset) then
                 nload_=nload_+meminset(i)
                 nradiate=nradiate+meminset(i)
-                exit
               endif
-            enddo
+            endif
           endif
         enddo
       elseif(textpart(1)(1:8).eq.'*RESTART') then
@@ -1796,7 +1858,7 @@ c     !
      &         ne1d,ne2d,nflow,set,meminset,rmeminset,jobnamec,
      &         irestartstep,icntrl,ithermal,nener,nstate_,ntie_,
      &         nslavs,nkon_,mcs,nprop_,mortar,ifacecount,nintpoint,
-     &         infree,nef,mpcend)
+     &         infree,nef,mpcend,nheading_,network,nfc_,ndc_,iprestr)
           irstrt(1)=-1
           nbounold=nboun_
           nforcold=nforc_
@@ -1841,12 +1903,12 @@ c     !
             noset(81:81)=' '
             ipos=index(noset,' ')
             noset(ipos:ipos)='N'
-            do i=1,nset_
+            call cident81(set,noset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.noset) then
                 nboun_=nboun_+ibound*meminset(i)
-                exit
               endif
-            enddo
+            endif
           endif
         enddo
       elseif(textpart(1)(1:10).eq.'*RIGIDBODY') then
@@ -1872,23 +1934,25 @@ c     !
           endif
         enddo
         if(noset(1:1).ne.' ') then
-          do i=1,nset_
+          call cident81(set,noset,nset_,i)
+          if(i.gt.0) then
             if(set(i).eq.noset) then
               nk_=nk_+2+meminset(i)
               nmpc_=nmpc_+3*meminset(i)
               memmpc_=memmpc_+18*meminset(i)
               nboun_=nboun_+3*meminset(i)
             endif
-          enddo
+          endif
         elseif(elset(1:1).ne.' ') then
-          do i=1,nset_
+          call cident81(set,elset,nset_,i)
+          if(i.gt.0) then
             if(set(i).eq.elset) then
               nk_=nk_+2+20*meminset(i)
               nmpc_=nmpc_+60*meminset(i)
               memmpc_=memmpc_+360*meminset(i)
               nboun_=nboun_+60*meminset(i)
             endif
-          enddo
+          endif
         endif
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &       ipoinp,inp,ipoinpc)
@@ -1915,9 +1979,13 @@ c     !
             elset(81:81)=' '
             ipos=index(elset,' ')
             elset(ipos:ipos)='E'
-            do js=1,nset_
-              if(set(js).eq.elset) exit
-            enddo
+            call cident81(set,elset,nset_,id)
+            js=nset_+1
+            if(id.gt.0) then
+              if(set(id).eq.elset) then
+                js=id
+              endif
+            endif
           endif
         enddo
         if(composite) then
@@ -2007,6 +2075,7 @@ c     !
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &       ipoinp,inp,ipoinpc)
       elseif(textpart(1)(1:9).eq.'*SUBMODEL') then
+        nsubmodel=nsubmodel+1
         ntie_=ntie_+1
         nam_=nam_+1
         namtot_=namtot_+4
@@ -2020,16 +2089,16 @@ c     !
             mastset(81:81)=' '
             ipos=index(mastset,' ')
             mastset(ipos:ipos)='E'
-            do i=1,nset_
-              if(set(i).eq.mastset) exit
-            enddo
+            call cident81(set,mastset,nset_,id)
+            i=nset_+1
+            if(id.gt.0) then
+              if(set(id).eq.mastset) then
+                i=id
+              endif
+            endif
             if(i.le.nset_) then
-              nset_=nset_+1
-              do k=1,81
-                set(nset_)(k:k)=' '
-              enddo
-              meminset(nset_)=meminset(nset_)+meminset(i)
-              rmeminset(nset_)=rmeminset(nset_)+meminset(i)
+              meminset(i)=meminset(i)+meminset(i)
+              rmeminset(i)=rmeminset(i)+meminset(i)
             endif
           elseif(textpart(j)(1:5).eq.'TYPE=') then
             if(textpart(j)(6:12).eq.'SURFACE') then
@@ -2042,8 +2111,37 @@ c     !
 !     
 !     local node or element face set
 !     
+        submset(1:8)='SUBMODEL'
+        if(nsubmodel.lt.10) then
+          submset(9:10)='00'
+          write(submset(11:11),'(i1)') nsubmodel
+        elseif(nsubmodel.lt.100) then
+          submset(9:9)='0'
+          write(submset(10:11),'(i2)') nsubmodel
+        elseif(nsubmodel.lt.1000) then
+          write(submset(9:11),'(i3)') nsubmodel
+        else
+          write(*,*) '*ERROR reading *SUBMODEL: no more than 999'
+          write(*,*) '       submodels allowed'
+          ier=1
+          return
+        endif
+        submset(12:12)=selabel
+        do i=13,81
+          submset(i:i)=' '
+        enddo
+!        
+        call cident81(set,submset,nset_,id)
         nset_=nset_+1
-        set(nset_)(1:1)=' '
+        do j=nset_,id+2,-1
+          meminset(j)=meminset(j-1)
+          rmeminset(j)=rmeminset(j-1)
+          set(j)=set(j-1)
+        enddo
+        js=id+1
+        set(js)=submset
+        meminset(js)=0
+        rmeminset(js)=0
         do
           call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
      &         ipoinp,inp,ipoinpc)
@@ -2054,35 +2152,47 @@ c     !
             noset(81:81)=' '
             ipos=index(noset,' ')
             noset(ipos:ipos)=selabel
-            do i=1,nset_-1
+            call cident81(set,noset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.noset) then
-                meminset(nset_)=meminset(nset_)+meminset(i)
+                meminset(js)=meminset(js)+meminset(i)
 !     
 !     surfaces are stored in expanded form 
 !     (no equivalent to generate)
 !     
-                rmeminset(nset_)=rmeminset(nset_)+meminset(i)
+                rmeminset(js)=rmeminset(js)+meminset(i)
               endif
-            enddo
+            endif
           else
-            meminset(nset_)=meminset(nset_)+1
-            rmeminset(nset_)=rmeminset(nset_)+1
+            meminset(js)=meminset(js)+1
+            rmeminset(js)=rmeminset(js)+1
           endif
         enddo
       elseif(textpart(1)(1:9).eq.'*SURFACE ') then
-        nset_=nset_+1
         sulabel='T'
         do i=2,n
           if(textpart(i)(1:5).eq.'NAME=')
      &         then
-            set(nset_)=textpart(i)(6:85)
-            set(nset_)(81:81)=' '
+            surfset=textpart(i)(6:85)
+            surfset(81:81)=' '
           elseif(textpart(i)(1:9).eq.'TYPE=NODE') then
             sulabel='S'
           endif
         enddo
-        ipos=index(set(nset_),' ')
-        set(nset_)(ipos:ipos)=sulabel
+        ipos=index(surfset,' ')
+        surfset(ipos:ipos)=sulabel
+        call cident81(set,surfset,nset_,id)
+        nset_=nset_+1
+        do j=nset_,id+2,-1
+          meminset(j)=meminset(j-1)
+          rmeminset(j)=rmeminset(j-1)
+          set(j)=set(j-1)
+        enddo
+        js=id+1
+        set(js)=surfset
+        meminset(js)=0
+        rmeminset(js)=0
+!        
         if(sulabel.eq.'S') then
           selabel='N'
         else
@@ -2098,26 +2208,27 @@ c     !
             noset(81:81)=' '
             ipos=index(noset,' ')
             noset(ipos:ipos)=selabel
-            do i=1,nset_-1
+            call cident81(set,noset,nset_,i)
+            if(i.gt.0) then
               if(set(i).eq.noset) then
-                meminset(nset_)=meminset(nset_)+meminset(i)
+                meminset(js)=meminset(js)+meminset(i)
 !     
 !     surfaces are stored in expanded form 
 !     (no equivalent to generate)
 !     
-                rmeminset(nset_)=rmeminset(nset_)+meminset(i)
+                rmeminset(js)=rmeminset(js)+meminset(i)
               endif
-            enddo
+            endif
           else
-            meminset(nset_)=meminset(nset_)+1
-            rmeminset(nset_)=rmeminset(nset_)+1
+            meminset(js)=meminset(js)+1
+            rmeminset(js)=rmeminset(js)+1
           endif
         enddo
 !     
 !     for CFD-calculations: local coordinate systems are
 !     stored as distributed load
 !     
-        if(icfd>0) nload_=nload_+rmeminset(nset_)
+        if(icfd>0) nload_=nload_+rmeminset(js)
       elseif(textpart(1)(1:16).eq.'*SURFACEBEHAVIOR') then
         ncmat_=max(4,ncmat_)
         ntmat_=max(1,ntmat_)
@@ -2199,21 +2310,35 @@ c     !
         islavset=0
         imastset=0
 !     
-        do i=1,nset_
+        call cident81(set,slavsets,nset_,i)
+        if(i.gt.0) then
           if(set(i).eq.slavsets) then
             islavset=i
             multslav=1
-          elseif(set(i).eq.slavsett) then
+          endif
+        endif
+        call cident81(set,slavsett,nset_,i)
+        if(i.gt.0) then
+          if(set(i).eq.slavsett) then
             islavset=i
             multslav=8
-          elseif(set(i).eq.mastsets) then
+          endif
+        endif
+        call cident81(set,mastsets,nset_,i)
+        if(i.gt.0) then
+          if(set(i).eq.mastsets) then
             imastset=i
             multmast=1
-          elseif(set(i).eq.mastsett) then
+          endif
+        endif
+        call cident81(set,mastsett,nset_,i)
+        if(i.gt.0) then
+          if(set(i).eq.mastsett) then
             imastset=i
             multmast=8
           endif
-        enddo
+        endif
+!
         if((islavset.ne.0).and.(imastset.ne.0)) then
           ncs_=ncs_+max(multslav*meminset(islavset),
      &         multmast*meminset(imastset))
@@ -2226,6 +2351,8 @@ c     !
           write(*,*) '       master set:',mastset(1:iposm-1)
           call inputerror(inpc,ipoinpc,iline,
      &         "*TIE%",ier)
+          call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &         ipoinp,inp,ipoinpc)
           cycle loop
         endif
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
@@ -2305,12 +2432,12 @@ c     !
           endif
         enddo
         if(surface(1:1).ne.' ') then
-          do i=1,nset_
+          call cident81(set,surface,nset_,i)
+          if(i.gt.0) then
             if(set(i).eq.surface) then
               nload_=nload_+meminset(i)
-              exit
             endif
-          enddo
+          endif
         endif
         do
           call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
@@ -2348,6 +2475,26 @@ c     !
      &           inl,ipoinp,inp,ipoinpc)
           enddo
         enddo
+      elseif(textpart(1)(1:12).eq.'*USERSECTION') then
+        nconstants=0
+        do i=2,n
+          if(textpart(i)(1:10).eq.'CONSTANTS=') then
+            read(textpart(i)(11:20),'(i10)',iostat=istat) 
+     &           nconstants
+            if(istat.gt.0) then
+              call inputerror(inpc,ipoinpc,iline,
+     &             "*USER SECTION%",ier)
+              exit
+            endif
+            nprop_=nprop_+nconstants
+            exit
+          endif
+        enddo
+        do
+          call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
+     &         ipoinp,inp,ipoinpc)
+          if((istat.lt.0).or.(key.eq.1)) exit
+        enddo
       else
 !     
         call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
