@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2020 Guido Dhondt
+!              Copyright (C) 1998-2021 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -36,9 +36,9 @@
 !
       integer ncont,ntie,i,j,k,nset,istartset(*),iendset(*),ialset(*),
      &  imast,nelem,jface,ncone,islav,ismallsliding,ipos,mortar,istep,
-     &  kflag,idummy,jact
+     &  kflag,idummy,jact,id
 !
-      real*8 tietol(3,*)
+      real*8 tietol(4,*)
 !
 !     number of master triangles
 !
@@ -64,9 +64,16 @@
 !
 !           determining the master surface
 !
-            do j=1,nset
-               if(set(j).eq.mastset) exit
-            enddo
+c            do j=1,nset
+c               if(set(j).eq.mastset) exit
+c            enddo
+            call cident81(set,mastset,nset,id)
+            j=nset+1
+            if(id.gt.0) then
+              if(mastset.eq.set(id)) then
+                j=id
+              endif
+            endif
             if(j.gt.nset) then
                ipos=index(mastset,' ')
                write(*,*) '*ERROR in allocont: master surface ',
@@ -132,31 +139,31 @@
 !
 !              face-to-face penalty contact (facial slave surface)
 !
-               mortar=1
+c               mortar=1
                nodeslavsurf=.false.
             elseif(slavset(ipos:ipos).eq.'M') then
 !
 !              quad-quad Mortar contact (facial slave surface)
 !
-               mortar=2
+c               mortar=2
                nodeslavsurf=.false.
             elseif(slavset(ipos:ipos).eq.'P') then
 !
 !              quad-lin Petrov Galerkin Mortar contact (facial slave surface)
 !
-               mortar=4
+c               mortar=4
                nodeslavsurf=.false.
             elseif(slavset(ipos:ipos).eq.'G') then
 !
 !              quad-quad Petrov Galerkin Mortar contact (facial slave surface)
 !
-               mortar=5
+c               mortar=5
                nodeslavsurf=.false.
             elseif(slavset(ipos:ipos).eq.'O') then
 !
 !              quad-lin Mortar contact (facial slave surface)
 !
-               mortar=3
+c               mortar=3
                nodeslavsurf=.false.
             else
 !
@@ -168,9 +175,16 @@
 !
 !           determining the slave surface
 !
-            do j=1,nset
-               if(set(j).eq.slavset) exit
-            enddo
+c            do j=1,nset
+c               if(set(j).eq.slavset) exit
+c            enddo
+            call cident81(set,slavset,nset,id)
+            j=nset+1
+            if(id.gt.0) then
+              if(slavset.eq.set(id)) then
+                j=id
+              endif
+            endif
             if(j.gt.nset) then
                if(mortar.eq.1) then
                   write(*,*) 
