@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                   */
-/*              Copyright (C) 1998-2020 Guido Dhondt                          */
+/*              Copyright (C) 1998-2021 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -74,15 +74,16 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	    char *tieset,double *cs,ITG *nintpoint,ITG *mortar,
 	    ITG *ifacecount,ITG **islavsurfp,double **pslavsurfp,
 	    double **clearinip,ITG *nmat,char *typeboun,
-	    ITG *ielprop,double *prop,char *orname,ITG *inewton){
+	    ITG *ielprop,double *prop,char *orname,ITG *inewton,
+	    double *t0g,double *t1g){
 
   /* calls the Arnoldi Package (ARPACK) */
   
   char bmat[2]="G", which[3]="LM", howmny[2]="A", fneig[132]="",
-    description[13]="            ",*lakon=NULL,jobnamef[396]="";
+    description[13]="            ",*lakon=NULL,jobnamef[396]="",*labmpc2=NULL;
 
   ITG *inum=NULL,k,ido,ldz,iparam[11],ipntr[14],lworkl,ngraph=1,im,
-    info,rvec=1,*select=NULL,lfin,j,lint,iout,ielas=0,icmd=0,mt=mi[1]+1,
+    info,rvec=1,*select=NULL,lfin,j,lint,iout,ielas=1,icmd=0,mt=mi[1]+1,
     iinc=1,nev,ncv,mxiter,jrow,ifreebody,kmax,
     mass[2]={1,1}, stiffness=1, buckling=0, rhsi=0, intscheme=0,noddiam=-1,
     coriolis=0,symmetryflag=0,inputformat=0,*ipneigh=NULL,*neigh=NULL,ne0,
@@ -94,7 +95,11 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
     icfd=0,*inomat=NULL,*ipkon=NULL,*kon=NULL,*ielmat=NULL,*ielorien=NULL,
     *islavact=NULL,maxprevcontel,iex,nslavs_prev_step,icutb=0,
     iflagact=0,*islavsurfold=NULL,ialeatoric=0,kscale=1,network=0,
-    *iponoel=NULL,*inoel=NULL,nrhs=1,igreen=0,node,idir,mscalmethod=0;
+    *iponoel=NULL,*inoel=NULL,nrhs=1,igreen=0,node,idir,mscalmethod=0,
+    *jqw=NULL,*iroww=NULL,nzsw,
+    *islavelinv=NULL,*irowtloc=NULL,*jqtloc=NULL,nboun2,
+    *ndirboun2=NULL,*nodeboun2=NULL,nmpc2,*ipompc2=NULL,*nodempc2=NULL,
+    *ikboun2=NULL,*ilboun2=NULL,*ikmpc2=NULL,*ilmpc2=NULL,mortartrafoflag=0;
 
   double *stn=NULL,*v=NULL,*resid=NULL,*z=NULL,*workd=NULL,
     *workl=NULL,*d=NULL,sigma=1,*temp_array=NULL,*pslavsurfold=NULL,
@@ -110,13 +115,14 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
     *di=NULL,sigmai=0,*workev=NULL,*ener=NULL,*xstate=NULL,*dc=NULL,
     *au=NULL,*ad=NULL,*b=NULL,*aub=NULL,*adb=NULL,*pslavsurf=NULL,
     *pmastsurf=NULL,*cdn=NULL,*energyini=NULL,*energy=NULL,
-    *cdnr=NULL,*cdni=NULL,*eme=NULL,alea=0.1,*smscale=NULL,zmax;
+    *cdnr=NULL,*cdni=NULL,*eme=NULL,alea=0.1,*smscale=NULL,zmax,*auw=NULL,
+    *autloc=NULL,*xboun2=NULL,*coefmpc2=NULL;
 
   FILE *f1;
 
   /* dummy arguments for the results call */
 
-  double *veold=NULL,*accold=NULL,bet,gam,dtime,time=1.;
+  double *veold=NULL,*accold=NULL,bet,gam,dtime=1.,time=1.;
 
 #ifdef SGI
   ITG token;
@@ -187,14 +193,6 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
   /* assigning the body forces to the elements */ 
 
   if(*nbody>0){
-    /*    ifreebody=*ne+1;
-    NNEW(ipobody,ITG,2*ifreebody**nbody);
-    for(k=1;k<=*nbody;k++){
-      FORTRAN(bodyforce,(cbody,ibody,ipobody,nbody,set,istartset,
-			 iendset,ialset,&inewton,nset,&ifreebody,&k));
-      RENEW(ipobody,ITG,2*(*ne+ifreebody));
-    }
-    RENEW(ipobody,ITG,2*(ifreebody-1));*/
     if(*inewton==1){
       printf("*ERROR in arpack: generalized gravity loading is not allowed in frequency calculations");
       FORTRAN(stop,());
@@ -339,7 +337,8 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	      itiefac,areaslav,iponoels,inoels,springarea,tietol,&reltime,
 	      imastnode,nmastnode,xmastnor,filab,mcs,ics,&nasym,
 	      xnoels,mortar,pslavsurf,pmastsurf,clearini,&theta,
-	      xstateini,xstate,nstate_,&icutb,&ialeatoric,jobnamef,&alea);
+	      xstateini,xstate,nstate_,&icutb,&ialeatoric,jobnamef,&alea,
+	      auw,jqw,iroww,&nzsw);
 	  
       printf("number of contact spring elements=%" ITGFORMAT "\n\n",*ne-ne0);
 	  
@@ -382,6 +381,14 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
     NNEW(qfx,double,3*mi[0]**ne);
   }
   NNEW(inum,ITG,*nk);
+
+  /* following fields are needed if *ener==1 or kode<-100 */
+    
+  NNEW(vini,double,mt**nk);
+  NNEW(stiini,double,6*mi[0]*ne0);
+  NNEW(emeini,double,6*mi[0]*ne0);
+  NNEW(enerini,double,mi[0]*ne0);
+    
   if(*iperturb==0){
     results(co,nk,kon,ipkon,lakon,ne,v,stn,inum,stx,
 	    elcon,nelcon,rhcon,nrhcon,alcon,nalcon,alzero,ielmat,
@@ -401,11 +408,15 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	    mortar,islavact,cdn,islavnode,nslavnode,ntie,clearini,
 	    islavsurf,ielprop,prop,energyini,energy,&kscale,iponoel,
 	    inoel,nener,orname,&network,ipobody,xbody,ibody,typeboun,
-	    itiefac,tieset,smscale,&mscalmethod,nbody);
+	    itiefac,tieset,smscale,&mscalmethod,nbody,t0g,t1g,
+	    islavelinv,autloc,irowtloc,jqtloc,&nboun2,
+	    ndirboun2,nodeboun2,xboun2,&nmpc2,ipompc2,nodempc2,coefmpc2,
+	    labmpc2,ikboun2,ilboun2,ikmpc2,ilmpc2,&mortartrafoflag,
+	    &intscheme);
   }else{
     results(co,nk,kon,ipkon,lakon,ne,v,stn,inum,stx,
 	    elcon,nelcon,rhcon,nrhcon,alcon,nalcon,alzero,ielmat,
-	    ielorien,norien,orab,ntmat_,t0,t1old,ithermal,
+	    ielorien,norien,orab,ntmat_,t1old,t1old,ithermal,
 	    prestr,iprestr,filab,eme,emn,een,iperturb,
 	    f,fn,nactdof,&iout,qa,vold,b,nodeboun,
 	    ndirboun,xboun,nboun,ipompc,
@@ -421,8 +432,15 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	    mortar,islavact,cdn,islavnode,nslavnode,ntie,clearini,
 	    islavsurf,ielprop,prop,energyini,energy,&kscale,iponoel,
 	    inoel,nener,orname,&network,ipobody,xbody,ibody,typeboun,
-	    itiefac,tieset,smscale,&mscalmethod,nbody);
+	    itiefac,tieset,smscale,&mscalmethod,nbody,t0g,t1g,
+	    islavelinv,autloc,irowtloc,jqtloc,&nboun2,
+	    ndirboun2,nodeboun2,xboun2,&nmpc2,ipompc2,nodempc2,coefmpc2,
+	    labmpc2,ikboun2,ilboun2,ikmpc2,ilmpc2,&mortartrafoflag,
+	    &intscheme);
   }
+  
+  SFREE(stiini);SFREE(emeini);SFREE(enerini);SFREE(vini);
+    
   SFREE(f);SFREE(v);SFREE(fn);SFREE(stx);SFREE(eme);
   if(*ithermal>1)SFREE(qfx);SFREE(inum);
   iout=1;
@@ -430,7 +448,7 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
   /* for the frequency analysis linear strain and elastic properties
      are used */
 
-  iperturb[1]=0;ielas=1;
+  iperturb[1]=0;
 
   /* filling in the matrix */
 
@@ -459,7 +477,8 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 		 xstateini,xstate,thicke,integerglob,doubleglob,
 		 tieset,istartset,iendset,ialset,ntie,&nasym,pslavsurf,
 		 pmastsurf,mortar,clearini,ielprop,prop,&ne0,fnext,&kscale,
-		 iponoel,inoel,&network,ntrans,inotr,trab,smscale,&mscalmethod);
+		 iponoel,inoel,&network,ntrans,inotr,trab,smscale,&mscalmethod,
+		 set,nset,islavelinv,autloc,irowtloc,jqtloc,&mortartrafoflag);
   }
   else{
     mafillsmmain(co,nk,kon,ipkon,lakon,ne,nodeboun,ndirboun,xboun,nboun,
@@ -478,7 +497,8 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 		 xstateini,xstate,thicke,integerglob,doubleglob,
 		 tieset,istartset,iendset,ialset,ntie,&nasym,pslavsurf,
 		 pmastsurf,mortar,clearini,ielprop,prop,&ne0,fnext,&kscale,
-		 iponoel,inoel,&network,ntrans,inotr,trab,smscale,&mscalmethod);
+		 iponoel,inoel,&network,ntrans,inotr,trab,smscale,&mscalmethod,
+		 set,nset,islavelinv,autloc,irowtloc,jqtloc,&mortartrafoflag);
 
     if(nasym==1){
       RENEW(au,double,nzs[2]+nzs[1]);
@@ -503,26 +523,7 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 		     xstateini,xstate,thicke,
 		     integerglob,doubleglob,tieset,istartset,iendset,
 		     ialset,ntie,&nasym,pslavsurf,pmastsurf,mortar,clearini,
-		     ielprop,prop,&ne0,&kscale,iponoel,inoel,&network);
-	  
-            /*FORTRAN(mafillsmas,(co,nk,kon,ipkon,lakon,ne,nodeboun,
-			  ndirboun,xboun,nboun,
-			  ipompc,nodempc,coefmpc,nmpc,nodeforc,ndirforc,xforc,
-			  nforc,nelemload,sideload,xload,nload,xbody,ipobody,
-			  nbody,cgr,ad,au,fext,nactdof,icol,jq,irow,neq,nzl,
-			  nmethod,ikmpc,ilmpc,ikboun,ilboun,
-			  elcon,nelcon,rhcon,nrhcon,alcon,nalcon,alzero,
-			  ielmat,ielorien,norien,orab,ntmat_,
-			  t0,t1old,ithermal,prestr,iprestr,vold,iperturb,sti,
-			  nzs,stx,adb,aub,iexpl,plicon,nplicon,plkcon,nplkcon,
-			  xstiff,npmat_,&dtime,matname,mi,
-			  ncmat_,mass,&stiffness,&buckling,&rhsi,&intscheme,
-			  physcon,shcon,nshcon,cocon,ncocon,ttime,&time,istep,&iinc,
-			  &coriolis,ibody,xloadold,&reltime,veold,springarea,nstate_,
-			  xstateini,xstate,thicke,
-			  integerglob,doubleglob,tieset,istartset,iendset,
-			  ialset,ntie,&nasym,pslavsurf,pmastsurf,mortar,clearini,
-			  ielprop,prop,&ne0,&kscale,iponoel,inoel,&network));*/
+		     ielprop,prop,&ne0,&kscale,iponoel,inoel,&network,set,nset);
     }
   }
 
@@ -546,7 +547,7 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	mi,sti,vr,vi,stnr,stni,vmax,stnmax,&ngraph,veold,ener,ne,
 	cs,set,nset,istartset,iendset,ialset,eenmax,fnr,fni,emn,
 	thicke,jobnamec,output,qfx,cdn,mortar,cdnr,cdni,nmat,
-	ielprop,prop);
+	ielprop,prop,sti);
     
     if(strcmp1(&filab[1044],"ZZS")==0){SFREE(ipneigh);SFREE(neigh);}
     SFREE(inum);FORTRAN(stop,());
@@ -592,6 +593,16 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
     matrixstorage(ad,&au,adb,aub,&sigma,icol,&irow,&neq[1],&nzs[1],
 		  ntrans,inotr,trab,co,nk,nactdof,jobnamec,mi,ipkon,
                   lakon,kon,ne,mei,nboun,nmpc,cs,mcs,ithermal,nmethod);
+
+    strcpy(fneig,jobnamec);
+    strcat(fneig,".frd");
+    if((f1=fopen(fneig,"ab"))==NULL){
+      printf("*ERROR in frd: cannot open frd file for writing...");
+      exit(0);
+    }
+    fprintf(f1," 9999\n");
+    fclose(f1);
+    FORTRAN(stopwithout201,());
 #else
     printf("*ERROR in arpack: the MATRIXSTORAGE library is not linked\n\n");
     FORTRAN(stop,());
@@ -1044,12 +1055,14 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
     if(*nprint>0) FORTRAN(writehe,(&j));
 
     NNEW(eei,double,6*mi[0]*ne0);
-    if(*nener==1){
-      NNEW(stiini,double,6*mi[0]*ne0);
-      NNEW(emeini,double,6*mi[0]*ne0);
-      NNEW(enerini,double,mi[0]*ne0);}
 
-    //    memset(&v[0],0.,sizeof(double)*mt**nk);
+    /* following fields are needed if *ener==1 or kode<-100 */
+    
+    NNEW(vini,double,mt**nk);
+    NNEW(stiini,double,6*mi[0]*ne0);
+    NNEW(emeini,double,6*mi[0]*ne0);
+    NNEW(enerini,double,mi[0]*ne0);
+
     DMEMSET(v,0,mt**nk,0.);
     if(*iperturb==0){
       results(co,nk,kon,ipkon,lakon,ne,v,stn,inum,
@@ -1072,12 +1085,16 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	      mortar,islavact,cdn,islavnode,nslavnode,ntie,clearini,
 	      islavsurf,ielprop,prop,energyini,energy,&kscale,iponoel,
               inoel,nener,orname,&network,ipobody,xbody,ibody,typeboun,
-	      itiefac,tieset,smscale,&mscalmethod,nbody);}
+	      itiefac,tieset,smscale,&mscalmethod,nbody,t0g,t1g,
+	      islavelinv,autloc,irowtloc,jqtloc,&nboun2,
+	      ndirboun2,nodeboun2,xboun2,&nmpc2,ipompc2,nodempc2,coefmpc2,
+	      labmpc2,ikboun2,ilboun2,ikmpc2,ilmpc2,&mortartrafoflag,
+	      &intscheme);}
     else{
       results(co,nk,kon,ipkon,lakon,ne,v,stn,inum,
 	      stx,elcon,
 	      nelcon,rhcon,nrhcon,alcon,nalcon,alzero,ielmat,ielorien,
-	      norien,orab,ntmat_,t0,t1old,ithermal,
+	      norien,orab,ntmat_,t1old,t1old,ithermal,
 	      prestr,iprestr,filab,eme,emn,een,iperturb,
 	      f,fn,nactdof,&iout,qa,vold,&z[lint],
 	      nodeboun,ndirboun,xboun,nboun,ipompc,
@@ -1093,11 +1110,14 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	      mortar,islavact,cdn,islavnode,nslavnode,ntie,clearini,
 	      islavsurf,ielprop,prop,energyini,energy,&kscale,iponoel,
               inoel,nener,orname,&network,ipobody,xbody,ibody,typeboun,
-	      itiefac,tieset,smscale,&mscalmethod,nbody);
+	      itiefac,tieset,smscale,&mscalmethod,nbody,t0g,t1g,
+	      islavelinv,autloc,irowtloc,jqtloc,&nboun2,
+	      ndirboun2,nodeboun2,xboun2,&nmpc2,ipompc2,nodempc2,coefmpc2,
+	      labmpc2,ikboun2,ilboun2,ikmpc2,ilmpc2,&mortartrafoflag,
+	      &intscheme);
     }
     SFREE(eei);
-    if(*nener==1){
-      SFREE(stiini);SFREE(emeini);SFREE(enerini);}
+    SFREE(stiini);SFREE(emeini);SFREE(enerini);SFREE(vini);
 
     ++*kode;
     if(igreen==0){
@@ -1123,7 +1143,7 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	  mi,stx,vr,vi,stnr,stni,vmax,stnmax,&ngraph,veold,ener,ne,
 	  cs,set,nset,istartset,iendset,ialset,eenmax,fnr,fni,emn,
 	  thicke,jobnamec,output,qfx,cdn,mortar,cdnr,cdni,nmat,
-	  ielprop,prop);
+	  ielprop,prop,sti);
     }else{
       frd(co,nk,kon,ipkon,lakon,ne,v,stn,inum,nmethod,
 	  kode,filab,een,t1old,fn,&freq,epn,ielmat,matname,enern,xstaten,
@@ -1132,7 +1152,7 @@ void arpack(double *co, ITG *nk, ITG **konp, ITG **ipkonp, char **lakonp,
 	  mi,stx,vr,vi,stnr,stni,vmax,stnmax,&ngraph,veold,ener,ne,
 	  cs,set,nset,istartset,iendset,ialset,eenmax,fnr,fni,emn,
 	  thicke,jobnamec,output,qfx,cdn,mortar,cdnr,cdni,nmat,
-	  ielprop,prop);
+	  ielprop,prop,sti);
     }
 
     if(strcmp1(&filab[1044],"ZZS")==0){SFREE(ipneigh);SFREE(neigh);}
