@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2020 Guido Dhondt                          */
+/*              Copyright (C) 1998-2021 Guido Dhondt                     */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -62,16 +62,16 @@ int main(int argc,char *argv[])
     *ncocon=NULL,*ibody=NULL,*ielprop=NULL,*islavsurf=NULL,
     *ipoinpc=NULL,mt,nxstate,nload0,iload,*iuel=NULL,*ne2boun=NULL,
     *irandomtype=NULL,irobustdesign[3],*iparentel=NULL,ifreebody,
-    *ipobody=NULL,inewton=0;
+    *ipobody=NULL,inewton=0,*iprfn=NULL,*konrfn=NULL;
      
   ITG nk,ne,nboun,nmpc,nforc,nload,nprint,nset,nalset,nentries=18,
-    nmethod,neq[3],i,mpcfree,mei[4],j,nzl,nam,nbounold,
+    nmethod,neq[3],i,mpcfree,mei[4]={0,0,0,0},j,nzl,nam,nbounold,
     nforcold,nloadold,nbody,nbody_,nbodyold,network,nheading_,
     k,nzs[3],nmpc_,nload_,nforc_,istep,istat,nboun_,nintpoint,
     iperturb[2],nmat,ntmat_,norien,ithermal[2]={0,0},nmpcold,
     iprestr,kode,isolver,nslavs,nkon_,ne0,nkon0,mortar,
     jout[2],nlabel,nkon,idrct,jmax[2],iexpl,nevtot,ifacecount,
-    iplas,npmat_,mi[3],ntrans,mpcend,namtot_,iumat,
+    iplas,npmat_,mi[3],ntrans,mpcend,namtot_,iumat,iheading,
     icascade,maxlenmpc,mpcinfo[4],ne1d,ne2d,infree[4],
     callfrommain,nflow,jin=0,irstrt[2],nener,jrstrt,nenerold,
     nline,*ipoinp=NULL,*inp=NULL,ntie,ntie_,mcs,nprop_,
@@ -79,7 +79,8 @@ int main(int argc,char *argv[])
     nmethodl,iaxial,inext,icontact,nobject,nobject_,iit,
     nzsprevstep[3],memmpcref_,mpcfreeref,maxlenmpcref,*nodempcref=NULL,
     *ikmpcref=NULL,isens,namtot,nstam,ndamp,nef,inp_size,
-    *ipoinp_sav=NULL,*inp_sav=NULL,irefineloop=0;
+    *ipoinp_sav=NULL,*inp_sav=NULL,irefineloop=0,icoordinate=0,
+    *nodedesi=NULL,ndesi=0,nobjectstart=0,nfc_,ndc_,nfc,ndc,*ikdc=NULL;
 
   ITG *meminset=NULL,*rmeminset=NULL;
 
@@ -96,12 +97,12 @@ int main(int argc,char *argv[])
     *xstate=NULL,*trab=NULL,*ener=NULL,*shcon=NULL,*cocon=NULL,
     *cs=NULL,*tietol=NULL,*fmpc=NULL,*prop=NULL,*t0g=NULL,*t1g=NULL,
     *xbody=NULL,*xbodyold=NULL,*coefmpcref=NULL,*dacon=NULL,*vel=NULL,
-    *velo=NULL,*veloo=NULL,energy[5];
+    *velo=NULL,*veloo=NULL,energy[5],*ratiorfn=NULL,*dgdxglob=NULL,
+    *g0=NULL,*xdesi=NULL,*coeffc=NULL,*edc=NULL;
     
   double ctrl[57];
 
-  double fei[3],*xmodal=NULL,timepar[5],
-    alpha[2],ttime,qaold[2],physcon[14];
+  double fei[3],*xmodal=NULL,timepar[5],alpha[2],ttime,qaold[2],physcon[14];
 
   double totalCalculixTime;
 
@@ -116,10 +117,10 @@ int main(int argc,char *argv[])
   if(argc==1){printf("Usage: CalculiX.exe -i jobname\n");FORTRAN(stop,());}
   else{
     for(i=1;i<argc;i++){
-      if(strcmp1(argv[i],"-i")==0) {
+      if(strcmp1(argv[i],"-i")==0){
 	strcpy(jobnamec,argv[i+1]);strcpy1(jobnamef,argv[i+1],132);jin++;break;}
-      if(strcmp1(argv[i],"-v")==0) {
-	printf("\nThis is Version 2.17\n\n");
+      if(strcmp1(argv[i],"-v")==0){
+	printf("\nThis is Version 2.19\n\n");
 	FORTRAN(stop,());
       }
     }
@@ -128,7 +129,7 @@ int main(int argc,char *argv[])
     /* next lines deactivated on March 18, 2020 */
     
     /*    for(i=1;i<argc;i++){
-      if(strcmp1(argv[i],"-o")==0) {
+      if(strcmp1(argv[i],"-o")==0){
 	strcpy(output,argv[i+1]);break;}
 	}*/
   }
@@ -144,12 +145,12 @@ int main(int argc,char *argv[])
   FORTRAN(openfile,(jobnamef));
 
   printf("\n************************************************************\n\n");
-  printf("CalculiX Version 2.17, Copyright(C) 1998-2020 Guido Dhondt\n");
+  printf("CalculiX Version 2.19, Copyright(C) 1998-2021 Guido Dhondt\n");
   printf("CalculiX comes with ABSOLUTELY NO WARRANTY. This is free\n");
   printf("software, and you are welcome to redistribute it under\n");
   printf("certain conditions, see gpl.htm\n\n");
   printf("************************************************************\n\n");
-  printf("You are using an executable made on Thu Jul 23 21:43:50 CEST 2020\n");
+  printf("You are using an executable made on Fri Dec 17 13:15:26 CET 2021\n");
   fflush(stdout);
 
   NNEW(ipoinp,ITG,2*nentries);
@@ -179,7 +180,8 @@ int main(int argc,char *argv[])
 	  &ndamp,&nef,&nk_,&ne_,&nalset_,&nmat_,&norien_,&nam_,
 	  &ntrans_,&ncs_,&nstate_,&ncmat_,&memmpc_,&nprint_,energy,ctrl,alpha,
 	  qaold,physcon,&istep,&istat,&iprestr,&kode,&nload,&nbody,&nforc,
-	  &nboun,&nk,&nmpc,&nam,&nzs_,&nlabel,&ttime);
+	  &nboun,&nk,&nmpc,&nam,&nzs_,&nlabel,&ttime,&iheading,&nfc,&nfc_,&ndc,
+	  &ndc_);
   
   NNEW(set,char,81*nset_);
   NNEW(meminset,ITG,nset_);
@@ -194,12 +196,14 @@ int main(int argc,char *argv[])
 		      &nprop_,ipoinpc,&nevdamp_,&npt_,&nslavs,&nkon_,&mcs,
 		      &mortar,&ifacecount,&nintpoint,infree,&nheading_,
 		      &nobject_,iuel,&iprestr,&nstam,&ndamp,&nef,&nbounold,
-		      &nforcold,&nloadold,&nbodyold,&mpcend,irobustdesign));
+		      &nforcold,&nloadold,&nbodyold,&mpcend,irobustdesign,
+		      &nfc_,&ndc_));
 
-  SFREE(set);SFREE(meminset);SFREE(rmeminset);mt=mi[1]+1;
+  //  SFREE(set);
+  SFREE(meminset);SFREE(rmeminset);mt=mi[1]+1;
   NNEW(heading,char,66*nheading_);
 
-  while(istat>=0) {
+  while(istat>=0){
 
     fflush(stdout);
 
@@ -209,7 +213,7 @@ int main(int argc,char *argv[])
 
     nzs[1]=nzs_;
 
-    if((istep==0)||(irstrt[0]<0)) {
+    if((istep==0)||(irstrt[0]<0)){
       ne=ne_;
       nset=nset_;
       nalset=nalset_;
@@ -274,6 +278,14 @@ int main(int argc,char *argv[])
       NNEW(ilmpc,ITG,nmpc_);
       NNEW(fmpc,double,nmpc_);
 
+      /* coupling, distributed */
+
+      if(nfc_>0){
+	NNEW(coeffc,double,7*nfc_);
+	NNEW(ikdc,ITG,ndc_);
+	NNEW(edc,double,12*ndc_);
+      }
+      
       /* nodal loads */
 
       NNEW(nodeforc,ITG,2*nforc_);
@@ -307,8 +319,8 @@ int main(int argc,char *argv[])
 
       /* set definitions */
 
-      NNEW(set,char,81*nset);
-      NNEW(istartset,ITG,nset);
+      RENEW(set,char,81*nset);
+      NNEW(istartset,ITG,nset);DMEMSET(istartset,0,nset,1);
       NNEW(iendset,ITG,nset);
       NNEW(ialset,ITG,nalset);
 
@@ -369,7 +381,7 @@ int main(int argc,char *argv[])
 
       /* material orientation */
 
-      if((istep==0)||((irstrt[0]<0)&&(norien>0))) {
+      if((istep==0)||((irstrt[0]<0)&&(norien>0))){
 	NNEW(orname,char,80*norien);
 	NNEW(orab,double,7*norien);
 	NNEW(ielorien,ITG,mi[2]*ne_);
@@ -377,23 +389,23 @@ int main(int argc,char *argv[])
 
       /* transformations */
 
-      if((istep==0)||((irstrt[0]<0)&&(ntrans>0))) {
+      if((istep==0)||((irstrt[0]<0)&&(ntrans>0))){
 	NNEW(trab,double,7*ntrans);
 	NNEW(inotr,ITG,2*nk_);
       }
 
       /* amplitude definitions */
 
-      if((istep==0)||((irstrt[0]<0)&&(nam_>0))) {
+      if((istep==0)||((irstrt[0]<0)&&(nam_>0))){
 	NNEW(amname,char,80*nam_);
 	NNEW(amta,double,2*namtot_);
 	NNEW(namta,ITG,3*nam_);
       }
 
-      if((istep==0)||((irstrt[0]<0)&&(ithermal[0]>0))) {
+      if((istep==0)||((irstrt[0]<0)&&(ithermal[0]>0))){
 	NNEW(t0,double,nk_);
 	NNEW(t1,double,nk_);
-	if((ne1d!=0)||(ne2d!=0)){
+	if((ne1d!=0)||(ne2d!=0)||(nuel_!=0)){
 	  NNEW(t0g,double,2*nk_);
 	  NNEW(t1g,double,2*nk_);
 	}
@@ -431,17 +443,21 @@ int main(int argc,char *argv[])
 
       if(ntie_>0){
 	NNEW(tieset,char,243*ntie_);
-	NNEW(tietol,double,3*ntie_);
+	NNEW(tietol,double,4*ntie_);
 	NNEW(cs,double,17*ntie_);
       }
 
       /* objectives for sensitivity analysis */
 
       if(nobject_>0){
-	NNEW(objectset,char,324*nobject_);
-	for(i=0;i<324*nobject_;i++){objectset[i]=' ';}
+	NNEW(nodedesi,ITG,nk_);
+	NNEW(dgdxglob,double,nobject_*2*nk_);
+	NNEW(g0,double,nobject_);
+	NNEW(xdesi,double,3*nk_);
+	NNEW(objectset,char,405*nobject_);
+	for(i=0;i<405*nobject_;i++){objectset[i]=' ';}
       }
-    
+
       /* temporary fields for cyclic symmetry calculations */
 
       if((ncs_>0)||(npt_>0)){
@@ -510,7 +526,7 @@ int main(int argc,char *argv[])
       RENEW(xbodyold,double,7*nbody_);
       for(i=7*nbodyold;i<7*nbody_;i++) xbodyold[i]=0;
 
-      if(nam > 0) {
+      if(nam>0){
 	RENEW(iamforc,ITG,nforc_);
 	RENEW(iamload,ITG,2*nload_);
 	RENEW(iamboun,ITG,nboun_);
@@ -526,7 +542,7 @@ int main(int argc,char *argv[])
       RENEW(ilmpc,ITG,nmpc_);
       RENEW(fmpc,double,nmpc_);
 
-      if(ntrans > 0){
+      if(ntrans>0){
 	RENEW(inotr,ITG,2*nk_);DMEMSET(inotr,2*nk,2*nk_,0);
       }
 
@@ -535,11 +551,11 @@ int main(int argc,char *argv[])
       if(ithermal[0] != 0){
 	RENEW(t0,double,nk_);DMEMSET(t0,nk,nk_,0.);
 	RENEW(t1,double,nk_);DMEMSET(t1,nk,nk_,0.);
-	if((ne1d!=0)||(ne2d!=0)){
+	if((ne1d!=0)||(ne2d!=0)||(nuel_!=0)){
 	  RENEW(t0g,double,2*nk_);DMEMSET(t0g,2*nk,2*nk_,0.);
 	  RENEW(t1g,double,2*nk_);DMEMSET(t1g,2*nk,2*nk_,0.);
 	}
-	if(nam > 0) {RENEW(iamt1,ITG,nk_);}
+	if(nam>0){RENEW(iamt1,ITG,nk_);}
       }
 
     }
@@ -584,7 +600,7 @@ int main(int argc,char *argv[])
 
     /* reading the input file */
 
-    if(istep==0)mortar=-1;
+    //    if(istep==0)mortar=-2;
     FORTRAN(calinput,(co,&nk,kon,ipkon,lakon,&nkon,&ne,nodeboun,ndirboun,xboun,
 		      &nboun,ipompc,nodempc,coefmpc,&nmpc,&nmpc_,nodeforc,
 		      ndirforc,xforc,&nforc,&nforc_,nelemload,sideload,xload,
@@ -612,7 +628,8 @@ int main(int argc,char *argv[])
 		      iuel,&nuel_,nodempcref,coefmpcref,ikmpcref,&memmpcref_,
 		      &mpcfreeref,&maxlenmpcref,&memmpc_,&isens,&namtot,&nstam,
 		      dacon,vel,&nef,velo,veloo,ne2boun,itempuser,
-		      irobustdesign,irandomtype,randomval));
+		      irobustdesign,irandomtype,randomval,&nfc,&nfc_,coeffc,
+		      ikdc,&ndc,&ndc_,edc));
     
     SFREE(idefforc);SFREE(idefload);SFREE(idefbody);
 
@@ -631,7 +648,7 @@ int main(int argc,char *argv[])
       RENEW(ipobody,ITG,2*(ifreebody-1));
     }
     
-    if(irefineloop==1) {
+    if(irefineloop==1){
 
       /* in the last step refinement was requested and the mesh
          was appropriately refined; this mesh has to be read */
@@ -646,7 +663,9 @@ int main(int argc,char *argv[])
 		  &prestr,&vold,&veold,&ielmat,irobustdesign,&irandomtype,
 		  &randomval,&nalset,&nalset_,&nkon,xnor,&iaxial,
 		  &network,&nlabel,iuel,iperturb,&iprestr,&ntie,tieset,
-		  &iparentel,ikboun,&ifreebody,&ipobody,&nbody);
+		  &iparentel,ikboun,&ifreebody,&ipobody,&nbody,&iprfn,
+		  &konrfn,&ratiorfn,nodempcref,coefmpcref,&memmpcref_,
+		  &mpcfreeref,&maxlenmpcref,&maxlenmpc,&norien,tietol);
     }
 
 #ifdef CALCULIX_EXTERNAL_BEHAVIOURS_SUPPORT
@@ -655,22 +674,26 @@ int main(int argc,char *argv[])
     }
 #endif /* CALCULIX_EXTERNAL_BEHAVIOURS_SUPPORT */
   
-    if((istep==1)&&(mortar==-1)){mortar=0;}else{icontact=1;}
+    //    if((istep==1)&&(mortar==-1)){mortar=0;}else{icontact=1;}
 
     nload0=nload;
-    //SFREE(idefforc);SFREE(idefload);SFREE(idefbody);
 
-    if(nheading_>=0){
+    if(iheading==0){
+      writeheading(jobnamec,heading,&nheading_);
+      iheading=1;
+    }
+
+    /*    if(nheading_>=0){
       writeheading(jobnamec,heading,&nheading_);
       SFREE(heading);
       nheading_=-1;
-    }
+      }*/
 
     if((abs(nmethod)!=1)||(iperturb[0]<2))icascade=0;
 
     //    FORTRAN(writeboun,(nodeboun,ndirboun,xboun,typeboun,&nboun));
 
-    if(istep==1) {
+    if(istep==1){
 
       SFREE(iuel);
 
@@ -695,7 +718,7 @@ int main(int argc,char *argv[])
 
       /* residual stresses/strains */
 
-      if(iprestr==1) {
+      if(iprestr==1){
 	RENEW(prestr,double,6*mi[0]*ne);
 	for(i=0;i<ne;i++){
 	  for(j=0;j<mi[0];j++){
@@ -751,7 +774,6 @@ int main(int argc,char *argv[])
 
       /* property cards */
 
-      //      if(nprop>0){
       if(nprop_>0){
 	RENEW(ielprop,ITG,ne);
 	RENEW(prop,double,nprop);
@@ -759,6 +781,16 @@ int main(int argc,char *argv[])
 	SFREE(ielprop);SFREE(prop);
       }
 
+      /* coupling, distributed */
+
+      if(nfc>0){
+	RENEW(coeffc,double,7*nfc);
+	RENEW(ikdc,ITG,ndc);
+	RENEW(edc,double,12*ndc);
+      }else{
+	SFREE(coeffc);SFREE(ikdc);SFREE(edc);
+      }
+
       /* fields for 1-D and 2-D elements */
 
       if((ne1d!=0)||(ne2d!=0)){
@@ -825,7 +857,7 @@ int main(int argc,char *argv[])
 
       /* material orientation */
 
-      if(norien > 0) {
+      if(norien>0){
 	RENEW(orname,char,80*norien);
 	RENEW(ielorien,ITG,mi[2]*ne);
 	RENEW(orab,double,7*norien);
@@ -838,7 +870,7 @@ int main(int argc,char *argv[])
 
       /* amplitude definitions */
 
-      if(nam > 0) {
+      if(nam>0){
 	RENEW(amname,char,80*nam);
 	RENEW(namta,ITG,3*nam);
 	RENEW(amta,double,2*namta[3*nam-2]);
@@ -852,15 +884,14 @@ int main(int argc,char *argv[])
 	SFREE(iamboun);
       }
 
-      if(ntrans > 0){
+      if(ntrans>0){
 	RENEW(trab,double,7*ntrans);
       }
       else{SFREE(trab);SFREE(inotr);}
 
-      //      if(ithermal[0]==0){SFREE(t0);SFREE(t1);SFREE(t0g);SFREE(t1g);}
       if(ithermal[0]==0){
 	SFREE(t0);SFREE(t1);
-	if((ne1d!=0)||(ne2d!=0)){
+	if((ne1d!=0)||(ne2d!=0)||(nuel_!=0)){
 	  SFREE(t0g);SFREE(t1g);
 	}
       }
@@ -872,7 +903,6 @@ int main(int argc,char *argv[])
       }else if(npt_>0){
 	SFREE(ics);
       }
-      //      SFREE(dcs);
       if((ncs_>0)||(npt_>0)){
 	SFREE(dcs);
       }
@@ -883,6 +913,16 @@ int main(int argc,char *argv[])
 	SFREE(cs);
       }
 
+      /* check for design variables */
+      
+      for(i=0;i<ntie;i++){
+        if(strcmp1(&tieset[i*243+80],"D")==0){
+          if(strcmp1(&tieset[i*243],"COORDINATE")==0){
+	    icoordinate=1;
+	  }
+	}
+      }
+
     }else{
 
       /* reallocating space in all but the first step (>1) */
@@ -924,7 +964,7 @@ int main(int argc,char *argv[])
 	RENEW(t1old,double,nk);
       }
 
-      if(nam > 0) {
+      if(nam>0){
 	RENEW(amname,char,80*nam);
 	RENEW(namta,ITG,3*nam);
 	RENEW(amta,double,2*namta[3*nam-2]);
@@ -954,11 +994,11 @@ int main(int argc,char *argv[])
     if(ithermal[0] != 0){
       RENEW(t0,double,nk);
       RENEW(t1,double,nk);
-      if((ne1d!=0)||(ne2d!=0)){
+      if((ne1d!=0)||(ne2d!=0)||(nuel_!=0)){
 	RENEW(t0g,double,2*nk);
 	RENEW(t1g,double,2*nk);
       }
-      if(nam > 0) {RENEW(iamt1,ITG,nk);}
+      if(nam>0){RENEW(iamt1,ITG,nk);}
     }
 
     RENEW(nelemload,ITG,2*nload);
@@ -997,11 +1037,11 @@ int main(int argc,char *argv[])
       SFREE(veold);
     }
 
-    if((nmethod==4)&&(iperturb[0]>1)) {
+    if((nmethod==4)&&(iperturb[0]>1)){
       NNEW(accold,double,mt*nk);
     }
 
-    if(nam > 0) {
+    if(nam>0){
       RENEW(iamforc,ITG,nforc);
       RENEW(iamload,ITG,2*nload);
       RENEW(iamboun,ITG,nboun);
@@ -1050,7 +1090,7 @@ int main(int argc,char *argv[])
       for(i=mi[2]*ne0;i<mi[2]*ne;i++)ielmat[i]=1;
     }
 
-    if(ntrans > 0){
+    if(ntrans>0){
       RENEW(inotr,ITG,2*nk);
     }
   
@@ -1088,8 +1128,7 @@ int main(int argc,char *argv[])
  
     /* decascading MPC's only necessary if MPC's changed */
 
-    if(((istep==1)||(ntrans>0)||(mpcend<0)||(nk!=nkold)||(nmpc!=nmpcold))&&(icascade==0)) {
-      //  if(icascade==0) {
+    if(((istep==1)||(ntrans>0)||(mpcend<0)||(nk!=nkold)||(nmpc!=nmpcold))&&(icascade==0)){
 
       /* decascading the MPC's */
 
@@ -1157,6 +1196,8 @@ int main(int argc,char *argv[])
     /* nmethod=12: sensitivity analysis  */
     /* nmethod=13: Green function calculation */
     /* nmethod=14: Robustness w.r.t. to geometric tolerances */
+    /* nmethod=15: Crack propagation */
+    /* nmethod=16: Feasible direction based on sensitivity information */
 
     if((nmethod<=1)||(nmethod==11)||((iperturb[0]>1)&&(nmethod<8)))
       {
@@ -1190,8 +1231,8 @@ int main(int argc,char *argv[])
 		    prset,&nener,trab,inotr,&ntrans,fmpc,ipobody,ibody,xbody,
 		    &nbody,
 		    xbodyold,timepar,thicke,jobnamec,tieset,&ntie,&istep,&nmat,
-		    ielprop,prop,typeboun,&mortar,mpcinfo,tietol,ics,&icontact,
-		    orname,itempuser);
+		    ielprop,prop,typeboun,&mortar,mpcinfo,tietol,ics,
+		    orname,itempuser,t0g,t1g);
 
 	  for(i=0;i<3;i++){nzsprevstep[i]=nzs[i];}
 
@@ -1225,7 +1266,7 @@ int main(int argc,char *argv[])
 		    &nintpoint,&mortar,&ifacecount,typeboun,&islavsurf,
 		    &pslavsurf,&clearini,&nmat,xmodal,&iaxial,&inext,&nprop,
 		    &network,orname,vel,&nef,velo,veloo,energy,itempuser,
-		    ipobody,&inewton);
+		    ipobody,&inewton,t0g,t1g,&ifreebody);
 
 	  memmpc_=mpcinfo[0];mpcfree=mpcinfo[1];icascade=mpcinfo[2];
 	  maxlenmpc=mpcinfo[3];
@@ -1259,7 +1300,7 @@ int main(int argc,char *argv[])
 	       ibody,xbody,&nbody,thicke,&nslavs,tietol,&nkon,mpcinfo,
 	       &ntie,&istep,&mcs,ics,tieset,cs,&nintpoint,&mortar,&ifacecount,
 	       &islavsurf,&pslavsurf,&clearini,&nmat,typeboun,ielprop,prop,
-	       orname,&inewton);
+	       orname,&inewton,t0g,t1g);
 
 	memmpc_=mpcinfo[0];mpcfree=mpcinfo[1];icascade=mpcinfo[2];
 	maxlenmpc=mpcinfo[3];
@@ -1295,7 +1336,7 @@ int main(int argc,char *argv[])
 		 ibody,xbody,&nbody,&nevtot,thicke,&nslavs,tietol,mpcinfo,
 		 &ntie,&istep,tieset,&nintpoint,&mortar,&ifacecount,&islavsurf,
 		 &pslavsurf,&clearini,&nmat,typeboun,ielprop,prop,orname,
-		 &inewton);
+		 &inewton,t0g,t1g);
 
 	memmpc_=mpcinfo[0];mpcfree=mpcinfo[1];icascade=mpcinfo[2];
 	maxlenmpc=mpcinfo[3];
@@ -1325,7 +1366,7 @@ int main(int argc,char *argv[])
 	       set,&nset,istartset,iendset,ialset,&nprint,prlab,
 	       prset,&nener,&isolver,trab,inotr,&ntrans,&ttime,fmpc,ipobody,
 	       ibody,xbody,&nbody,thicke,jobnamec,&nmat,ielprop,prop,
-	       orname,typeboun);
+	       orname,typeboun,t0g,t1g);
 #else
       printf("*ERROR in CalculiX: the ARPACK library is not linked\n\n");
       FORTRAN(stop,());
@@ -1360,7 +1401,8 @@ int main(int argc,char *argv[])
 	     prset,&nener,trab,&inotr,&ntrans,&fmpc,ipobody,ibody,xbody,&nbody,
 	     xbodyold,&istep,&isolver,jq,output,&mcs,&nkon,&mpcend,ics,cs,
 	     &ntie,tieset,&idrct,jmax,ctrl,&itpamp,tietol,&nalset,
-	     ikforc,ilforc,thicke,&nslavs,&nmat,typeboun,ielprop,prop,orname);
+	     ikforc,ilforc,thicke,&nslavs,&nmat,typeboun,ielprop,prop,orname,
+	     t0g,t1g);
       }
     else if(nmethod==5)
       {
@@ -1397,7 +1439,7 @@ int main(int argc,char *argv[])
 		    &mpcend,
 		    ctrl,ikforc,ilforc,thicke,&nmat,typeboun,ielprop,prop,
 		    orname,
-		    &ndamp,dacon);
+		    &ndamp,dacon,t0g,t1g);
       }
     else if((nmethod==6)||(nmethod==7))
       {
@@ -1427,7 +1469,7 @@ int main(int argc,char *argv[])
 		    cs,
 		    &ntie,tieset,&idrct,jmax,ctrl,&itpamp,tietol,&nalset,
 		    ikforc,ilforc,thicke,jobnamef,mei,&nmat,ielprop,prop,orname,
-		    typeboun);
+		    typeboun,t0g,t1g);
       }
     else if((nmethod>7)&&(nmethod<12)){
 
@@ -1459,34 +1501,62 @@ int main(int argc,char *argv[])
 		       &ntie,&tieset,&itpamp,&iviewfile,jobnamec,&tietol,
 		       &nslavs,thicke,
 		       ics,&nalset,&nmpc_,&nmat,typeboun,&iaxial,&nload_,&nprop,
-		       &network,orname);
+		       &network,orname,t0g,t1g);
 
       memmpc_=mpcinfo[0];mpcfree=mpcinfo[1];icascade=mpcinfo[2];
       maxlenmpc=mpcinfo[3];
     }
 
     else if(nmethod==12){
-  
-      sensitivity(co,&nk,&kon,&ipkon,&lakon,&ne,nodeboun,ndirboun,
-		  xboun,&nboun,ipompc,nodempc,coefmpc,labmpc,&nmpc,nodeforc,
-		  ndirforc,xforc,&nforc,nelemload,sideload,xload,&nload,
-		  nactdof,icol,jq,&irow,neq,&nzl,&nmethod,ikmpc,
-		  ilmpc,ikboun,ilboun,elcon,nelcon,rhcon,nrhcon,
-		  alcon,nalcon,alzero,&ielmat,&ielorien,&norien,orab,&ntmat_,
-		  t0,t1,t1old,ithermal,prestr,&iprestr,vold,iperturb,sti,nzs,
-		  &kode,filab,eme,&iexpl,plicon,
-		  nplicon,plkcon,nplkcon,&xstate,&npmat_,matname,
-		  &isolver,mi,&ncmat_,&nstate_,cs,&mcs,&nkon,&ener,
-		  xbounold,xforcold,xloadold,amname,amta,namta,
-		  &nam,iamforc,iamload,iamt1,iamboun,&ttime,
-		  output,set,&nset,istartset,iendset,ialset,&nprint,prlab,
-		  prset,&nener,trab,inotr,&ntrans,fmpc,ipobody,ibody,xbody,&nbody,
-		  xbodyold,timepar,thicke,jobnamec,tieset,&ntie,&istep,&nmat,
-		  ielprop,prop,typeboun,&mortar,mpcinfo,tietol,ics,&icontact,
-		  &nobject,&objectset,&istat,orname,nzsprevstep,&nlabel,physcon,
-		  jobnamef,iponor,knor,&ne2d,iponoel,inoel,&mpcend);
+      
+      if(icoordinate==1){
+	sensi_coor(co,&nk,&kon,&ipkon,&lakon,&ne,nodeboun,ndirboun,
+		   xboun,&nboun,ipompc,nodempc,coefmpc,labmpc,&nmpc,nodeforc,
+		   ndirforc,xforc,&nforc,nelemload,sideload,xload,&nload,
+		   nactdof,icol,jq,&irow,neq,&nzl,&nmethod,ikmpc,
+		   ilmpc,ikboun,ilboun,elcon,nelcon,rhcon,nrhcon,
+		   alcon,nalcon,alzero,&ielmat,&ielorien,&norien,orab,&ntmat_,
+		   t0,t1,t1old,ithermal,prestr,&iprestr,vold,iperturb,sti,nzs,
+		   &kode,filab,eme,&iexpl,plicon,
+		   nplicon,plkcon,nplkcon,&xstate,&npmat_,matname,
+		   &isolver,mi,&ncmat_,&nstate_,cs,&mcs,&nkon,&ener,
+		   xbounold,xforcold,xloadold,amname,amta,namta,
+		   &nam,iamforc,iamload,iamt1,iamboun,&ttime,
+		   output,set,&nset,istartset,iendset,ialset,&nprint,prlab,
+		   prset,&nener,trab,inotr,&ntrans,fmpc,ipobody,ibody,xbody,
+		   &nbody,
+		   xbodyold,timepar,thicke,jobnamec,tieset,&ntie,&istep,&nmat,
+		   ielprop,prop,typeboun,&mortar,mpcinfo,tietol,ics,
+		   &nobject,&objectset,&istat,orname,nzsprevstep,&nlabel,
+		   physcon,
+		   jobnamef,iponor,knor,&ne2d,iponoel,inoel,&mpcend,dgdxglob,
+		   g0,&nodedesi,&ndesi,&nobjectstart,&xdesi);
+    
+      }else{
+        sensi_orien(co,&nk,&kon,&ipkon,&lakon,&ne,nodeboun,ndirboun,
+		    xboun,&nboun,ipompc,nodempc,coefmpc,labmpc,&nmpc,nodeforc,
+		    ndirforc,xforc,&nforc,nelemload,sideload,xload,&nload,
+		    nactdof,icol,jq,&irow,neq,&nzl,&nmethod,ikmpc,
+		    ilmpc,ikboun,ilboun,elcon,nelcon,rhcon,nrhcon,
+		    alcon,nalcon,alzero,&ielmat,&ielorien,&norien,orab,&ntmat_,
+		    t0,t1,t1old,ithermal,prestr,&iprestr,vold,iperturb,sti,nzs,
+		    &kode,filab,eme,&iexpl,plicon,
+		    nplicon,plkcon,nplkcon,&xstate,&npmat_,matname,
+		    &isolver,mi,&ncmat_,&nstate_,cs,&mcs,&nkon,&ener,
+		    xbounold,xforcold,xloadold,amname,amta,namta,
+		    &nam,iamforc,iamload,iamt1,iamboun,&ttime,
+		    output,set,&nset,istartset,iendset,ialset,&nprint,prlab,
+		    prset,&nener,trab,inotr,&ntrans,fmpc,ipobody,ibody,xbody,
+		    &nbody,
+		    xbodyold,timepar,thicke,jobnamec,tieset,&ntie,&istep,&nmat,
+		    ielprop,prop,typeboun,&mortar,mpcinfo,tietol,ics,
+		    &nobject,&objectset,&istat,orname,nzsprevstep,&nlabel,
+		    physcon,
+		    jobnamef,iponor,knor,&ne2d,iponoel,inoel,&mpcend);
+      }
+    
     }
-
+    
     else if(nmethod==14){
   
       robustdesign(co,&nk,&kon,&ipkon,&lakon,&ne,nodeboun,ndirboun,xboun,
@@ -1503,7 +1573,7 @@ int main(int argc,char *argv[])
 		   iendset,ialset,&nprint,prlab,prset,&nener,trab,inotr,
 		   &ntrans,fmpc,ipobody,ibody,xbody,&nbody,xbodyold,timepar,
 		   thicke,jobnamec,tieset,&ntie,&istep,&nmat,ielprop,prop,
-		   typeboun,&mortar,mpcinfo,tietol,ics,&icontact,&nobject,
+		   typeboun,&mortar,mpcinfo,tietol,ics,&nobject,
 		   &objectset,&istat,orname,nzsprevstep,&nlabel,physcon,
 		   jobnamef,iponor,knor,&ne2d,iponoel,inoel,&mpcend,
 		   irobustdesign,irandomtype,randomval);
@@ -1511,10 +1581,22 @@ int main(int argc,char *argv[])
 
     else if(nmethod==15){
       
-      /*   crackpropagation(&ipkon,&kon,&lakon,&ne,&nk,jobnamec,&nboun,iamboun,xboun,
+      crackpropagation(&ipkon,&kon,&lakon,&ne,&nk,jobnamec,&nboun,iamboun,xboun,
 		       &nload,sideload,iamload,&nforc,iamforc,xforc,ithermal,t1,
 		       iamt1,&co,&nkon,mi,&ielmat,matname,output,&nmat,set,
-		       &nset,istartset,iendset,ialset,jmax);*/
+		       &nset,istartset,iendset,ialset,jmax,timepar,nelcon,
+		       elcon,&ncmat_,&ntmat_,&istep,filab,&nmethod,mei);
+    }
+
+    else if(nmethod==16){
+    
+      feasibledirection(&nobject,&objectset,&dgdxglob,g0,&ndesi,nodedesi,&nk,
+			&isolver,&ipkon,&kon,&lakon,&ne,nelemload,&nload,
+			nodeboun,&nboun,ndirboun,ithermal,co,vold,mi,&ielmat,
+			ielprop,prop,&kode,&nmethod,filab,&nstate_,&istep,cs,
+			set,&nset,istartset,iendset,ialset,jobnamec,output,
+			&ntrans,inotr,trab,orname,xdesi);         
+      
     }
 
     SFREE(nactdof);SFREE(icol);SFREE(jq);SFREE(irow);SFREE(ipobody);
@@ -1531,7 +1613,7 @@ int main(int argc,char *argv[])
 	   finished and a refined mesh was created and stored.
 	   The calculation has to restart from the beginning with
 	   this new mesh */
-	
+       
 	memcpy(ipoinp,ipoinp_sav,sizeof(ITG)*2*nentries);
 	memcpy(inp,inp_sav,sizeof(ITG)*inp_size);
 
@@ -1558,7 +1640,9 @@ int main(int argc,char *argv[])
 		    &knor,&thicke,&offset,&iponoel,&inoel,&rig,
 		    &ne2boun,&islavsurf,&mortar,&pslavsurf,&clearini,
 		    &nobject_,&objectset,&nmethod,iperturb,&irefineloop,
-		    &iparentel);
+		    &iparentel,&iprfn,&konrfn,&ratiorfn,&heading,
+		    &nodedesi,&dgdxglob,&g0,&nuel_,&xdesi,&nfc,&coeffc,
+		    &ikdc,&edc);
 
 	/* closing and reopening the output files */
 	
@@ -1578,7 +1662,8 @@ int main(int argc,char *argv[])
 		&nk_,&ne_,&nalset_,&nmat_,&norien_,&nam_,&ntrans_,
 		&ncs_,&nstate_,&ncmat_,&memmpc_,&nprint_,energy,ctrl,alpha,
 		qaold,physcon,&istep,&istat,&iprestr,&kode,&nload,&nbody,&nforc,
-		&nboun,&nk,&nmpc,&nam,&nzs_,&nlabel,&ttime);
+		&nboun,&nk,&nmpc,&nam,&nzs_,&nlabel,&ttime,&iheading,&nfc,
+		&nfc_,&ndc,&ndc_);
   
 	NNEW(set,char,81*nset_);
 	NNEW(meminset,ITG,nset_);
@@ -1595,9 +1680,10 @@ int main(int argc,char *argv[])
 			    &mortar,&ifacecount,&nintpoint,infree,&nheading_,
 			    &nobject_,iuel,&iprestr,&nstam,&ndamp,&nef,
 			    &nbounold,&nforcold,&nloadold,&nbodyold,&mpcend,
-			    irobustdesign));
+			    irobustdesign,&nfc_,&ndc_));
 
-	SFREE(set);SFREE(meminset);SFREE(rmeminset);mt=mi[1]+1;
+	//	SFREE(set);
+	SFREE(meminset);SFREE(rmeminset);mt=mi[1]+1;
 	NNEW(heading,char,66*nheading_);
 
 	continue;
@@ -1610,18 +1696,18 @@ int main(int argc,char *argv[])
 
     /* deleting the perturbation loads and temperatures */
 
-    if((iperturb[0]==1)&&(nmethod==3)) {
+    if((iperturb[0]==1)&&(nmethod==3)){
       nforc=0;
       nload=0;
       nbody=0;
-      if(ithermal[0]==1) {
+      if(ithermal[0]==1){
 	for(k=0;k<nk;++k){
 	  t1[k]=t0[k];
 	}
       }
     }else{
       nbounold=nboun;
-      for (i=0;i<nboun;i++) {
+      for(i=0;i<nboun;i++){
 	nodebounold[i]=nodeboun[i];
 	ndirbounold[i]=ndirboun[i];
       }
@@ -1631,36 +1717,36 @@ int main(int argc,char *argv[])
       
       /* resetting the amplitude to none except for time=total time amplitudes */
 
-      if(nam > 0) {
-	for (i=0;i<nboun;i++) {
+      if(nam>0){
+	for(i=0;i<nboun;i++){
 	  if(iamboun[i]>0){
 	    if(namta[3*iamboun[i]-1]>0){
 	      iamboun[i]=0;
 	      xboun[i]=xbounold[i];}
 	  }
 	}
-	for (i=0;i<nforc;i++){
+	for(i=0;i<nforc;i++){
 	  if(iamforc[i]>0){
 	    if(namta[3*iamforc[i]-1]>0){
 	      iamforc[i]=0;
 	      xforc[i]=xforcold[i];}
 	  }
 	}
-	for (i=0;i<2*nload;i++){
+	for(i=0;i<2*nload;i++){
 	  if(iamload[i]>0){
 	    if(namta[3*iamload[i]-1]>0){
 	      iamload[i]=0;
 	      xload[i]=xloadold[i];}
 	  }
 	}
-	for (i=1;i<3*nbody;i=i+3){
+	for(i=1;i<3*nbody;i=i+3){
 	  if(ibody[i]>0){
 	    if(namta[3*ibody[i]-1]>0){
 	      ibody[i]=0;
 	      xbody[7*(i-1)/3]=xbodyold[7*(i-1)/3];}
 	  }
 	}
-	if(ithermal[0]==1) {
+	if(ithermal[0]==1){
 	  if(iamt1[i]>0){
 	    if(namta[3*iamt1[i]-1]>0){
 	      iamt1[i]=0;
@@ -1728,7 +1814,8 @@ int main(int argc,char *argv[])
 			      tietol,&nslavs,t0g,t1g,&nprop,ielprop,prop,
 			      &mortar,&nintpoint,&ifacecount,islavsurf,
 			      pslavsurf,clearini,irstrt,vel,&nef,velo,veloo,
-			      ne2boun,&memmpc_));
+			      ne2boun,&memmpc_,heading,&nheading_,&network,
+			      &nfc,&ndc,coeffc,ikdc,edc));
       }
     } 
 	  
@@ -1772,74 +1859,9 @@ int main(int argc,char *argv[])
 	      &knor,&thicke,&offset,&iponoel,&inoel,&rig,
 	      &ne2boun,&islavsurf,&mortar,&pslavsurf,&clearini,
 	      &nobject_,&objectset,&nmethod,iperturb,&irefineloop,
-	      &iparentel);
-  
-  /************************
-  
-  if(ncs_>0) SFREE(ics);
-  if(mcs>0) SFREE(cs);
-  SFREE(tieset);SFREE(tietol);
-
-  SFREE(co);SFREE(kon);SFREE(ipkon);SFREE(lakon);
-
-  SFREE(nodeboun);SFREE(ndirboun);SFREE(typeboun);SFREE(xboun);SFREE(ikboun);
-  SFREE(ilboun);SFREE(nodebounold);SFREE(ndirbounold);SFREE(xbounold);
-
-  SFREE(ipompc);SFREE(labmpc);SFREE(ikmpc);SFREE(ilmpc);SFREE(fmpc);
-  SFREE(nodempc);SFREE(coefmpc);
-
-  SFREE(nodempcref);SFREE(coefmpcref);SFREE(ikmpcref);
-
-  SFREE(nodeforc);SFREE(ndirforc);SFREE(xforc);SFREE(ikforc);SFREE(ilforc);
-  SFREE(xforcold);
-
-  SFREE(nelemload);SFREE(sideload);SFREE(xload);SFREE(xloadold);
-
-  SFREE(cbody);SFREE(ibody);SFREE(xbody);SFREE(xbodyold);
-
-  if(nam>0){SFREE(iamboun);SFREE(iamforc);SFREE(iamload);SFREE(amname);
-    SFREE(amta);SFREE(namta);}
-
-  SFREE(set);SFREE(istartset);SFREE(iendset);SFREE(ialset);
-
-  SFREE(elcon);SFREE(nelcon);SFREE(rhcon);SFREE(nrhcon);SFREE(shcon);
-  SFREE(nshcon);
-  SFREE(cocon);SFREE(ncocon);SFREE(alcon);SFREE(nalcon);SFREE(alzero);
-  if(nprop>0){SFREE(ielprop);SFREE(prop);}
-  if(npmat_>0){SFREE(plicon);SFREE(nplicon);SFREE(plkcon);SFREE(nplkcon);}
-  if(ndamp>0){SFREE(dacon);}
-
-  if(norien>0){SFREE(orname);SFREE(orab);SFREE(ielorien);}
-  if(ntrans>0){SFREE(trab);SFREE(inotr);}
-  if(iprestr>0){SFREE(prestr);}
-
-  if(ithermal[0]!=0){
-    SFREE(t0);SFREE(t1);SFREE(t1old);
-    if(nam>0) SFREE(iamt1);
-    if((ne1d!=0)||(ne2d!=0)){SFREE(t0g);SFREE(t1g);}
-  }
-
-  if(irobustdesign[0]>0){SFREE(irandomtype);SFREE(randomval);}
-  
-  SFREE(prlab);SFREE(prset);SFREE(filab);SFREE(xmodal);
-
-  SFREE(ielmat);SFREE(matname);
-
-  SFREE(sti);SFREE(eme);SFREE(ener);SFREE(xstate);
-
-  SFREE(vold);SFREE(veold);SFREE(vel);SFREE(velo);SFREE(veloo);
-
-  if((ne1d!=0)||(ne2d!=0)){
-    SFREE(iponor);SFREE(xnor);SFREE(knor);SFREE(thicke);SFREE(offset);
-    SFREE(iponoel);SFREE(inoel);SFREE(rig);SFREE(ne2boun);
-  }
-
-  SFREE(islavsurf);
-  if(mortar==1){SFREE(pslavsurf);SFREE(clearini);}
-
-  if(nobject_>0){SFREE(objectset);}
-
-  ******************/
+	      &iparentel,&iprfn,&konrfn,&ratiorfn,&heading,
+	      &nodedesi,&dgdxglob,&g0,&nuel_,&xdesi,&nfc,&coeffc,
+	      &ikdc,&edc);
   
 #ifdef CALCULIX_MPI
   MPI_Finalize();
