@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2020 Guido Dhondt
+!              Copyright (C) 1998-2021 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 !
       subroutine deformationplasticitys(inpc,textpart,elcon,nelcon,
      &  nmat,ntmat_,ncmat_,irstrt,istep,istat,n,iperturb,iline,ipol,
-     &  inl,ipoinp,inp,ipoinpc,ier)
+     &  inl,ipoinp,inp,ipoinpc,ier,iplas)
 !
 !     reading the input deck: *DEFORMATION PLASTICITY
 !
@@ -29,7 +29,7 @@
 !
       integer nelcon(2,*),nmat,ntmat,ntmat_,istep,istat,ier,
      &  n,key,i,iperturb(*),iend,ncmat_,irstrt(*),iline,ipol,inl,
-     &  ipoinp(2,*),inp(3,*),ipoinpc(0:*)
+     &  ipoinp(2,*),inp(3,*),ipoinpc(0:*),iplas
 !
       real*8 elcon(0:ncmat_,ntmat_,*)
 !
@@ -39,6 +39,7 @@
       write(*,*) '*INFO reading *DEFORMATION PLASTICITY: nonlinear'
       write(*,*) '      geometric effects are turned on'
       write(*,*)
+      iplas=1
 !
       if((istep.gt.0).and.(irstrt(1).ge.0)) then
          write(*,*) '*ERROR reading *DEFORMATION PLASTICITY:'
