@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2020 Guido Dhondt                          */
+/*              Copyright (C) 1998-2021 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -22,7 +22,7 @@
 #include <pthread.h>
 #include "CalculiX.h"
 
-static char *lakon1,*sideload1,*matname1,*tieset1;
+static char *lakon1,*sideload1,*matname1,*tieset1,*set1;
 
 static ITG *nk1,*kon1,*ipkon1,*ne1,*nodeboun1,*ndirboun1,*nboun1,*ipompc1,
   *nodempc1,*nmpc1,*nodeforc1,*ndirforc1,*nforc1,*nelemload1,*nload1,*ipobody1,
@@ -33,7 +33,7 @@ static ITG *nk1,*kon1,*ipkon1,*ne1,*nodeboun1,*ndirboun1,*nboun1,*ipompc1,
   *intscheme1,*nshcon1,*ncocon1,*istep1,*iinc1,*coriolis1,*ibody1,*nstate1_,
   *integerglob1,*istartset1,*iendset1,*ialset1,*ntie1,*nasym1,*mortar1,
   *ielprop1,*ne01,*kscale1,*iponoel1,*inoel1,*network1,*neaparm=NULL,
-  *nebparm=NULL,*neapart=NULL,*nebpart=NULL;
+  *nebparm=NULL,*neapart=NULL,*nebpart=NULL,*nset1;
 
 static double *co1,*xboun1,*coefmpc1,*xforc1,*xload1,*xbody1,*cgr1,*ad1=NULL,
   *au1=NULL,*bb1,*elcon1,*rhcon1,*alcon1,*alzero1,*orab1,*t01,*t11,*prestr1,
@@ -69,7 +69,8 @@ void mafillsmasmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,ITG *ne,
 		ITG *istartset,ITG *iendset,ITG *ialset,ITG *ntie,
 		ITG *nasym,double *pslavsurf,double *pmastsurf,ITG *mortar,
 		double *clearini,ITG *ielprop,double *prop,ITG *ne0,
-		ITG *kscale,ITG *iponoel,ITG *inoel,ITG *network){
+		ITG *kscale,ITG *iponoel,ITG *inoel,ITG *network,
+		char *set,ITG *nset){
 
   ITG i,j,isiz,nec;
       
@@ -213,6 +214,7 @@ void mafillsmasmain(double *co,ITG *nk,ITG *kon,ITG *ipkon,char *lakon,ITG *ne,
   ntie1=ntie;nasym1=nasym;pslavsurf1=pslavsurf;pmastsurf1=pmastsurf;
   mortar1=mortar;clearini1=clearini;ielprop1=ielprop;prop1=prop;ne01=ne0;
   kscale1=kscale;iponoel1=iponoel;inoel1=inoel;network1=network;
+  set1=set;nset1=nset;
   
   /* calculating the stiffness/mass */
     
@@ -295,7 +297,7 @@ void *mafillsmasmt(ITG *i){
 		      integerglob1,doubleglob1,tieset1,istartset1,iendset1,
 		      ialset1,ntie1,nasym1,pslavsurf1,pmastsurf1,mortar1,
 		      clearini1,ielprop1,prop1,ne01,kscale1,iponoel1,inoel1,
-		      network1,&neam,&nebm,&neat,&nebt));
+		      network1,&neam,&nebm,&neat,&nebt,set1,nset1));
 
   return NULL;
 }
