@@ -1,5 +1,5 @@
 /* CalculiX - A 3-dimensional finite element program */
-/*Copyright (C) 1998-2020 Guido Dhondt*/
+/*Copyright (C) 1998-2021 Guido Dhondt*/
 
 /* This program is free software; you can redistribute it and/or */
 /* modify it under the terms of the GNU General Public License as*/
@@ -17,8 +17,11 @@
 
 #ifdef PASTIX
 
-#include <pastix.h>
 #include <spm.h>
+#include <pastix.h>
+//#include <api.h>
+//#include <nompi.h>
+//#include <datatypes.h>
 #include <sys/time.h>
 #include <stdio.h>
 #include <math.h>
@@ -41,9 +44,10 @@ ITG *icolTotal = NULL, *irowTotal = NULL;
 
 // Matrix data from previous iteration
 ITG neqPrev=0, nzsPrev=0;
-ITG *icolPrev=NULL,*irowPrev=NULL,*jqPrev=NULL;
+ITG *icolPrev=NULL,*irowPrev=NULL,*jqPrev=NULL,*offsetPrev=NULL;
 ITG inputformatPrev=-1;
 ITG basePrev=1;
+char noScale=0;
 
 // Current sparse matrix in CSC
 double *aupastix=NULL;
@@ -57,7 +61,10 @@ char redo = 1;
 #define AS 2
 char modePrev = BASIC;
 char mode = BASIC;
-char needCleanup=0;
+
+#define SINGLE_SOLVE 1;
+#define MULTI_SOLVE 2;
+char usage = SINGLE_SOLVE;
 
 // PaStiX configuration
 spm_int_t iparm_basic[IPARM_SIZE];
@@ -115,6 +122,7 @@ struct pastix_data_s {
     ITG *jqPrev;
     ITG inputformatPrev;
     ITG basePrev;
+    ITG *offsetPrev;
     double *aupastix;
     ITG *icolpastix;
     ITG *irowpastix;
@@ -139,13 +147,13 @@ struct pastix_data_s {
 typedef struct pastix_data_s pastix_data_object;
 
 pastix_data_object pastix_mode_basic = {
-    0,0,NULL,NULL,0,0,NULL,NULL,NULL,-1,0,
+    0,0,NULL,NULL,0,0,NULL,NULL,NULL,-1,0,NULL,
     NULL,NULL,NULL,1,0,0,0,1,0,0,0,NULL,NULL,iparm_basic,dparm_basic,
     0,1,NULL,NULL
     };
 
 pastix_data_object pastix_mode_as = {
-    0,0,NULL,NULL,0,0,NULL,NULL,NULL,-1,0,
+    0,0,NULL,NULL,0,0,NULL,NULL,NULL,-1,0,NULL,
     NULL,NULL,NULL,1,0,0,0,1,0,0,0,NULL,NULL,iparm_as,dparm_as,
     0,1,NULL,NULL
     };
@@ -159,6 +167,7 @@ void pastix_init(double *ad, double *au, double *adb, double *aub,
 	// if reusing, only update the value pointer of the sparse matrix
 	if(!redo){
 	    pastixResetSteps(pastix_data);
+        if(spm->values != aupastix && spm->values != NULL ) free(spm->values);
 		spm->values = aupastix;
 
 		printf("\n");
@@ -197,10 +206,10 @@ void pastix_init(double *ad, double *au, double *adb, double *aub,
 	// Set best PaStiX parameters for CalculiX usage
     iparm[IPARM_ORDERING]  				= PastixOrderScotch;
     if( mode == AS ){
-	    iparm[IPARM_SCHEDULER] 			= PastixSchedStatic;
+        iparm[IPARM_SCHEDULER] 			= PastixSchedStatic;
     }
     else{
-	    iparm[IPARM_SCHEDULER] 			= PastixSchedParsec;
+        iparm[IPARM_SCHEDULER] 			= PastixSchedParsec;
     }
 	iparm[IPARM_THREAD_NBR]				= nthread_mkl;
 	iparm[IPARM_GPU_NBR]   				= (int) gpu;
@@ -209,6 +218,8 @@ void pastix_init(double *ad, double *au, double *adb, double *aub,
 	iparm[IPARM_MAX_BLOCKSIZE] 			= 2048;
 	iparm[IPARM_FACTORIZATION] 			= PastixFactLU;
 	iparm[IPARM_TASKS2D_WIDTH] 			= globDoublePrecision ? 256 : 128;
+    iparm[IPARM_REFINEMENT]             = PastixRefineGMRES;
+
 
 	iparm[IPARM_REUSE_LU] 				= firstIter ? 0 : 1;
 	iparm[IPARM_REUSE_LU] 				= forceRedo ? 2 : 1;
@@ -216,34 +227,36 @@ void pastix_init(double *ad, double *au, double *adb, double *aub,
     iparm[IPARM_GPU_MEMORY_PERCENTAGE] 	= 95;
     iparm[IPARM_GPU_MEMORY_BLOCK_SIZE] 	= 64 * 1024;
 
-    
+
+    char usage_call = MULTI_SOLVE
+    if( usage == usage_call ){
+        dparm[DPARM_EPSILON_REFINEMENT] 	= 1e-7;
+        iparm[IPARM_ITERMAX]            	= 50;
+        iparm[IPARM_GMRES_IM]            	= 50;
+    } else{
     dparm[DPARM_EPSILON_REFINEMENT] 	= 1e-12;
-    iparm[IPARM_ITERMAX]            	= 50;
-    iparm[IPARM_GMRES_IM]            	= 50;
+    dparm[DPARM_EPSILON_MAGN_CTRL]  	= 0.;
+    iparm[IPARM_ITERMAX]            	= 70;
+    iparm[IPARM_GMRES_IM]            	= 70;
+    }
 
 	// Initialize sparse matrix
 	spm = malloc( sizeof( spmatrix_t ) );
 	spmInit(spm);
-	spm->flttype = globDoublePrecision ? SpmDouble : SpmFloat;
+    spm->flttype = globDoublePrecision ? SpmDouble : SpmFloat;
+    if(spm->values != aupastix && spm->values != NULL ) free(spm->values);
 	spm->values = aupastix;
 	spm->fmttype = SpmCSC;
 	spm->nexp = spm->gNexp = spm->gN = spm->n = *neq;
     spm->mtxtype = SpmGeneral;
     if( *inputformat == 3 ){
 	    spm->nnzexp = spm->gnnzexp = spm->gnnz = spm->nnz = nzsTotal + *neq;
-        spm->flttype = SpmDouble;
     } else{
 	    spm->nnzexp = spm->gnnzexp = spm->gnnz = spm->nnz = nzsTotal * 2 + *neq;
     }
 	spm->colptr = (spm_int_t*) icolpastix;
 	spm->rowptr = (spm_int_t*) irowpastix;
 
-    if(*inputformat == 3){
-        spmatrix_t *spm_tmp=NULL;
-        spm_tmp = spmCopy(spm);
-        spmCheckAndCorrect( spm_tmp, spm);
-    	spm->flttype = globDoublePrecision ? SpmDouble : SpmFloat;
-    }
 
 	// initialize pastix
 	pastixInit( &pastix_data, MPI_COMM_WORLD, iparm, dparm );
@@ -252,8 +265,10 @@ void pastix_init(double *ad, double *au, double *adb, double *aub,
 	spmPrintInfo( spm, stdout );
 	printf("\n");
 	
-	// perform reordering, analysis and symbolic factorization
-	pastix_task_analyze( pastix_data, spm );
+	// perform reordering, analysis and symbolic factorization if it's more than 1 equation
+    if(spm->n > 1){
+	    pastix_task_analyze( pastix_data, spm );
+    }
 }
 
 void pastix_csc_conversion(double *ad, double *au, double *adb, double *aub, 
@@ -263,14 +278,14 @@ double *sigma,ITG *icol, ITG *irow,
 	
 	ITG i,j;
     char merged=0;
-	
+
 	// jq for the merged matrix
 	ITG* jqTotal = NULL;
 	
-	redo = forceRedo ? 1 : 0;
-	
 	if(*neq != neqPrev || *inputformat != inputformatPrev)
-		redo = 1;
+		forceRedo = 1;
+	
+	redo = forceRedo ? 1 : 0;
 	
 	if(!redo){
 		nzsTotal = 0;
@@ -306,7 +321,7 @@ double *sigma,ITG *icol, ITG *irow,
 			}
 			icolTotal[i] = curColTotal;
 			nzsTotal += curColTotal;
-		}
+        }
 		
 		
 		// compute jq for the merged matrix
@@ -327,7 +342,7 @@ double *sigma,ITG *icol, ITG *irow,
 		}
 
 		// allocate space for the sparse matrix
-        if(*symmetryflag || *inputformat != 3)
+        if(*symmetryflag && *inputformat != 3)
 			NNEW(auPtr,double,2 * nzsTotal);
 		else
 			NNEW(auPtr,double,nzsTotal);
@@ -335,24 +350,18 @@ double *sigma,ITG *icol, ITG *irow,
 		NNEW(adPtr,double,neqPrev);
 		NNEW(irowTotal, ITG, nzsTotal);
 
-        if(*symmetryflag || *inputformat != 3){
+        if(*symmetryflag && *inputformat != 3){
             j=2*nzsTotal;
         }
         else{
             j=nzsTotal;
         }
-        #pragma omp parallel for
-        for(i=0;i<j;i++){
-            auPtr[i] = 0.0;
-        }
 
-        #pragma omp parallel for
-        for(i=0;i<*neq;i++){
-            adPtr[i] = 0.0;
-        }
+        memset(auPtr, 0, j * sizeof(double));
+        memset(adPtr, 0, *neq * sizeof(double));
 		
 		// merge the old and the new sparsity pattern
-		#pragma omp parallel for
+		#pragma omp parallel for shared(auPtr)
 		for(i=0;i<*neq;i++){
 			ITG kCur = jq[i] - base;
 			ITG kPrev = jqPrev[i] - base;
@@ -418,6 +427,9 @@ double *sigma,ITG *icol, ITG *irow,
 		SFREE(irowPrev);
 		SFREE(icolPrev);
         SFREE(jqPrev);
+		irowPrev = NULL;
+		icolPrev = NULL;
+        jqPrev = NULL;
 		
 		
 		// update pointers to the merged matrix
@@ -438,12 +450,19 @@ double *sigma,ITG *icol, ITG *irow,
 	else
 	{
 		// This is executed in either the first iteration, or when the number of equations changed
-	
+        
 		printf("Not reusing csc.\n");
 		if(icolPrev != NULL){
 			SFREE(icolPrev);
+            icolPrev = NULL;
+        }
+		if(irowPrev != NULL){
 			SFREE(irowPrev);
+            irowPrev = NULL;
+        }
+   		if(jqPrev != NULL){
             SFREE(jqPrev);
+            jqPrev = NULL;
 		}
 
 		NNEW(icolPrev,ITG,*neq);
@@ -474,18 +493,32 @@ double *sigma,ITG *icol, ITG *irow,
 			// We allocate 10% more space for the values than required so that we have to perform the expensive cudaMallocHost only once, even when the size of the matrix increases slightly
 			if((nzsTotal * 2 + *neq) > pastix_nnzBound){
 				// perform the call with PaStiX because pinned memory allocation via CUDA is performed if gpu is activated
+                if( !firstIter && aupastix == spm->values ) spm->values = NULL;
 				pastixAllocMemory((void**)&aupastix, sizeof(double) * 1.1 * (nzsTotal * 2 + *neq), gpu);
 				pastix_nnzBound = 1.1 * (nzsTotal * 2 + *neq);
 			}
-			NNEW(irowpastix,ITG,nzsTotal*2+*neq);
-			NNEW(icolpastix,ITG,*neq+1);
+            if(irowpastix != NULL ){
+                SFREE(irowpastix);
+                if( irowpastix == spm->rowptr ) spm->rowptr = NULL;
+            }
+
+  			NNEW(irowpastix,ITG,nzsTotal*2+*neq);
+
+            if(icolpastix != NULL ){
+                SFREE(icolpastix);
+                if( icolpastix == spm->colptr ) spm->colptr = NULL;
+            }
+
+   			NNEW(icolpastix,ITG,*neq+1);
 			
 			if(irowacc != NULL){
 				SFREE(irowacc);		
+                irowacc = NULL;
 			}
 			NNEW(irowacc,ITG,*neq);		
 			if(irowPrediction != NULL){
 				SFREE(irowPrediction);
+                irowPrediction = NULL;
 			}
 			NNEW(irowPrediction,ITG,nzsTotal);
 		}
@@ -506,11 +539,11 @@ double *sigma,ITG *icol, ITG *irow,
 		}
 		
 		// copy lower triangular values to the right position in the CSC
-		#pragma omp parallel for private(i, j)
+		#pragma omp parallel for private(j) shared(aupastix)
 		for(i=0;i<*neq;i++){
 			ITG k_pastix = icolpastix[i] + irowacc[i];
 			ITG k = jqTotal[i] - 1;
-			aupastix[k_pastix-1] = ad[i] - (merged != 0 ? 0 : (*sigma == 0 ? 0.0 : (*sigma)*adb[i]));
+			aupastix[k_pastix-1] = ad[i] - (merged != 0 ? 0.0 : (*sigma == 0.0 ? 0.0 : (*sigma)*adb[i]));
    			memcpy(aupastix + k_pastix, au + k, sizeof(double) * icol[i]);
             if(*sigma != 0.0 && !merged ){
                 for(j=0;j<icol[i];j++){
@@ -521,11 +554,11 @@ double *sigma,ITG *icol, ITG *irow,
 		}
 		
 		// copy the upper triangular values to the right position in the CSC
-		#pragma omp parallel for private(i, j)
+		#pragma omp parallel for private(j) shared(aupastix)
 		for(i=0;i<*neq;i++){
 			ITG k = jqTotal[i] - 1;
 			for(j=0;j<icol[i];j++){
-    			aupastix[irowPrediction[k] + icolpastix[irow[k]-1] - 1] = au[k+(*symmetryflag == 0 ? 0 : (*nzs == *nzs3 ? nzsTotal : *nzs3))] - (merged != 0 ? 0 : (*sigma == 0 ? 0.0 : (*sigma *aub[k+(*symmetryflag == 0 ? 0 : (*nzs == *nzs3 ? nzsTotal : *nzs3))])));
+    			aupastix[irowPrediction[k] + icolpastix[irow[k]-1] - 1] = au[k+(*symmetryflag == 0 ? 0 : (*nzs == *nzs3 ? nzsTotal : *nzs3))] - (merged != 0 ? 0 : (*sigma == 0.0 ? 0.0 : (*sigma *aub[k+(*symmetryflag == 0 ? 0 : (*nzs == *nzs3 ? nzsTotal : *nzs3))])));
 				k++;
 			}
 		}
@@ -541,50 +574,149 @@ double *sigma,ITG *icol, ITG *irow,
 				memcpy(irowpastix + k_pastix, irow + k, sizeof(ITG) * icol[i]);
 			}
 			
-			#pragma omp parallel for private(i, j)
+			#pragma omp parallel for private(j) shared(irowpastix)
 			for(i=0;i<*neq;i++){
 				ITG k = jqTotal[i] - 1;
 				for(j=0;j<icol[i];j++){
 					irowpastix[irowPrediction[k] + icolpastix[irow[k]-1] - 1] = i+1;
 					k++;
 				}
-			}
+            }
 		}
 	}
 	else if(*inputformat==3){
 
+
+        ITG countnew = 0;
+        ITG *row_newEntries = NULL;
+        ITG *col_newEntries = NULL;
+
+        // search for missing entries for a structural symmetric matrix
         if(redo){
-       		NNEW(irowpastix,ITG,nzsTotal+*neq);
-    		NNEW(icolpastix,ITG,*neq+1);
+            row_newEntries = malloc( sizeof(ITG) * nzsTotal );
+            col_newEntries = malloc( sizeof(ITG) * nzsTotal );
+            memset( row_newEntries, 0, sizeof(ITG) * nzsTotal );
+            memset( col_newEntries, 0, sizeof(ITG) * nzsTotal );
+            char found = 0;
+            ITG z = 0;
+            ITG temp = 0;
+    
+             // loop through the columns
+            #pragma omp parallel for private(j,z) firstprivate(temp,found)
+            for(i=0;i<*neq;i++){
+                // loop through the entries in this column
+                for(j=jqTotal[i]-1;j<jqTotal[i+1]-1;j++){
+                    temp = irow[j];
+                    // loop through the symmetric column counter part to check for symmetry
+                    for(z=jqTotal[temp-1]-1;z<jqTotal[temp]-1;z++){
+                        if( irow[z]-1 == i ){
+                            found=1;
+                            break;
+                        }
+                    }
+                    // if no entry was found add a dummy to the array and increase the counter for missing entries
+                    #pragma omp critical
+                    if( found == 0 ){
+                        row_newEntries[countnew] = i + 1;
+                        col_newEntries[countnew] = temp;
+                        countnew++;
+                    }
+                    found = 0;
+                }
+            }
+
+            printf("added %d entries to the matrix\n",countnew);
+            nzsTotal += countnew;
+
+            // allocate memory for the PaStiX arrays and free the old ones if necessary
        		if((nzsTotal + *neq) > pastix_nnzBound){
+                if( !firstIter && aupastix == spm->values ) spm->values = NULL;
             	pastixAllocMemory((void**)&aupastix, sizeof(double) * 1.1 * (nzsTotal + *neq), gpu);
    				pastix_nnzBound = 1.1 * (nzsTotal + *neq);
             }
+
+            memset( aupastix, 0, sizeof(double) * pastix_nnzBound );
+
+            if(irowpastix != NULL ){
+                SFREE(irowpastix);
+                if(irowpastix == spm->rowptr) spm->rowptr = NULL;
+            }
+
+  			NNEW(irowpastix,ITG,nzsTotal+*neq);
+
+            if(icolpastix != NULL ){
+                SFREE(icolpastix);
+                if(icolpastix == spm->colptr) spm->colptr = NULL;
+            }
+
+   			NNEW(icolpastix,ITG,*neq+1);
             memcpy(icolpastix, jqTotal, sizeof(ITG) * (*neq+1));
+
+            if(offsetPrev != NULL ){
+                SFREE(offsetPrev);
+            }
+            NNEW(offsetPrev,ITG,*neq+1);
+            memset(offsetPrev,0,*neq+1);
+        }
+        else{
+            nzsTotal += offsetPrev[*neq];
+            memset( aupastix, 0, sizeof(double) * pastix_nnzBound );
         }
 
-
-        ITG kflag,ilength;
-        kflag = 2;
-
-        #pragma omp parallel for private(i, j)
+        //#pragma omp parallel for private(j) firstprivate(offsetPrev) 
         for(i=0;i<*neq;i++){
+            ITG entriesPerColumn = jqTotal[i+1] - jqTotal[i];
+            ITG offsetSource = jqTotal[i] - jqTotal[0];
+
             if(redo){
-                memcpy(irowpastix + i + (jqTotal[i] - jqTotal[0]), irow + (jqTotal[i]-jqTotal[0]), sizeof(ITG) * (jqTotal[i+1] - jqTotal[i]));
-                irowpastix[i+jqTotal[i+1]-1] = i+1;
-                icolpastix[i+1] += i+1;
+                // copy irow column per column and add the additional diagonal entry
+                memcpy(irowpastix + i + offsetSource + offsetPrev[i], irow + offsetSource,
+                        sizeof(ITG) * entriesPerColumn);
+                irowpastix[i+jqTotal[i+1]-jqTotal[0]+offsetPrev[i]] = i+1;
             }
-            memcpy(aupastix + i + (jqTotal[i] - jqTotal[0]), au + (jqTotal[i]-jqTotal[0]), sizeof(double) * (jqTotal[i+1] - jqTotal[i]));
+        
+            // copy au column per column
+            memcpy(aupastix + i + offsetSource + offsetPrev[i], au + offsetSource,
+                    sizeof(double) * entriesPerColumn);
+            
+            // subtract the buckling values
             if(*sigma != 0 && merged == 0){
-                for(j=0;j<jqTotal[i+1]-jqTotal[i];j++){
-                    aupastix[i+(jqTotal[i]-jqTotal[0])+j] -= (*sigma)*aub[jqTotal[i]-jqTotal[0]+j];
+                for(j=0;j<entriesPerColumn;j++){
+                    aupastix[i + offsetSource + j + offsetPrev[i]] -= (*sigma)*aub[offsetSource + j];
                 }
             }
-            aupastix[i+jq[i+1]-1] = ad[i] - (merged != 0 ? 0.0 : (*sigma == 0 ? 0.0 : (*sigma)*adb[i]));
 
+            // add the diagonal entries to aupastix
+            aupastix[i + (jqTotal[i+1] - jqTotal[0]) + offsetPrev[i]] = ad[i] - (merged != 0 ? 0.0 : (*sigma == 0 ? 0.0 : (*sigma)*adb[i]));
+
+
+            // add the found entries for making the matrix structural symmetric and increase the resulting offset in arrays
+            if(redo){
+                offsetPrev[i+1] = offsetPrev[i];
+                for( j=0;j<countnew;j++ ){
+                    if( col_newEntries[j]-1 == i ){
+                        irowpastix[i + (jqTotal[i+1] - jqTotal[0]) + 1 + offsetPrev[i+1]] = row_newEntries[j];
+                        offsetPrev[i+1]++;
+                    }
+                }
+                // add the diagonal and additional entries to the column pointer
+                icolpastix[i+1] += i+1+offsetPrev[i+1];
+            }
+                        
         }
 
         if(redo) icolpastix[*neq] = nzsTotal + *neq + 1;
+
+        // free arrays for added symmetrized entries
+        if(row_newEntries){
+            SFREE(row_newEntries);
+            row_newEntries = NULL;
+        }
+        if(col_newEntries){
+            SFREE(col_newEntries);
+            col_newEntries = NULL;
+        }
+
 	} 
 
 
@@ -604,17 +736,15 @@ double *sigma,ITG *icol, ITG *irow,
 
 // PaStiX invocation when the factorization function is called individually
 void pastix_factor_main_generic(double *ad, double *au, double *adb, double *aub, 
-double *sigma,ITG *icol, ITG *irow, 
+        double *sigma,ITG *icol, ITG *irow, 
 		ITG *neq, ITG *nzs, ITG *symmetryflag, ITG *inputformat,
 		ITG *jq, ITG *nzs3){
 			
     pastix_set_globals(mode);	
-    gpu = 0;
 	// Set GPU flag from environment
-//	const char* pastix_gpu = getenv("PASTIX_GPU");
-//	if(pastix_gpu)
-//		//gpu = (*pastix_gpu == '1') ? 1 : 0;
-//		gpu = ( mode == AS ) ? 0 : (*pastix_gpu == '1') ? 1 : 0;
+	const char* pastix_gpu = getenv("PASTIX_GPU");
+	if(pastix_gpu)
+		gpu = ( mode == AS ) ? 0 : (*pastix_gpu == '1') ? 1 : 0;
 	
 	// Perform individual invocations always in double precision. If previous iterations were in single precision, do not reuse.
 	forceRedo=1;
@@ -623,7 +753,10 @@ double *sigma,ITG *icol, ITG *irow,
 	// invoke PaStiX
 	pastix_csc_conversion(ad, au, adb, aub, sigma, icol, irow, neq, nzs, symmetryflag, inputformat, jq, nzs3);
 	pastix_cleanup(neq,symmetryflag);
-	pastix_init(ad, au, adb, aub, sigma, icol, irow, neq, nzs, symmetryflag, inputformat, jq, nzs3);	
+	pastix_init(ad, au, adb, aub, sigma, icol, irow, neq, nzs, symmetryflag, inputformat, jq, nzs3);
+    gpu = 0;
+    iparm[IPARM_GPU_NBR]=0;
+    pastix_data->iparm[IPARM_GPU_NBR]=0;
 	pastix_factor(ad, au, adb, aub, sigma, icol, irow, neq, nzs, symmetryflag, inputformat, jq, nzs3);
 }
 
@@ -645,6 +778,7 @@ ITG pastix_solve_generic(double *x, ITG *neq,ITG *symmetryflag,ITG *nrhs){
 	ITG i;
 	double* b;
 	float* buffer;
+    ITG rc=0;
 	
 	// dont call pastix with only one equation, might lead to segfault
 	if(spm->n == 1)
@@ -678,23 +812,52 @@ ITG pastix_solve_generic(double *x, ITG *neq,ITG *symmetryflag,ITG *nrhs){
 		#pragma omp parallel for
 		for(i = 0; i < (*nrhs) * (*neq); i++){ 
 			buffer[i] = (float) x[i];
-		}
+        }
 		
-		pastix_task_solve( pastix_data, *nrhs, buffer, spm->n );
+		rc = pastix_task_solve( pastix_data, *nrhs, buffer, spm->n );
 
 		#pragma omp parallel for
 		for(i = 0; i < (*nrhs) * (*neq); i++){
 			x[i] = (double) buffer[i];
-		}
+        }
 		SFREE(buffer);
+        buffer = NULL;
 	}
 	else{
-		pastix_task_solve( pastix_data, *nrhs, x, spm->n );
+		rc = pastix_task_solve( pastix_data, *nrhs, x, spm->n );
 	}
+
+    // check for NaN in the solution
+    if( x[0] != x[0] ){
+        printf("\nSolution contains NaN!\n\n");
+        if( noScale ){
+            return -1;
+        }
+        else{
+            return -2;
+        }
+    }
 	
 	// invoke iterative refinement in double precision
-	ITG rc = pastix_task_refine( pastix_data, spm->n, *nrhs, (void*)b, spm->n, (void*)x, spm->n );
+    //dparm[DPARM_EPSILON_MAGN_CTRL] = 1e-4;
+    //iparm[IPARM_ITERMAX]           = 3;
 
+    char usage_call = SINGLE_SOLVE
+    if( usage == usage_call || rc != 0 ){
+        // invoke iterative refinement in double precision
+        rc = pastix_task_refine( pastix_data, spm->n, *nrhs, (void*)b, spm->n, (void*)x, spm->n );
+
+    	iparm[IPARM_GPU_NBR] 		   = 0;
+        dparm[DPARM_EPSILON_MAGN_CTRL] = 1e-14;
+        iparm[IPARM_ITERMAX]           = 50;
+  	    rc = pastix_task_refine( pastix_data, spm->n, *nrhs, (void*)b, spm->n, (void*)x, spm->n );
+    	iparm[IPARM_GPU_NBR] 		   = (int) gpu;
+        iparm[IPARM_ITERMAX]           = 70;
+    } else{
+        rc = pastix_task_refine( pastix_data, spm->n, *nrhs, (void*)b, spm->n, (void*)x, spm->n );
+    }
+    //    dparm[DPARM_EPSILON_MAGN_CTRL] = 1e-14;
+    //    iparm[IPARM_ITERMAX]           = 70;
 //    FILE *f=fopen("spm.out","a");
 //    fprintf(f,"\n\nMatrix\n");
 //    spmConvert(SpmCSR, spm);
@@ -714,14 +877,20 @@ ITG pastix_solve_generic(double *x, ITG *neq,ITG *symmetryflag,ITG *nrhs){
 //    fclose(f);
 
 	SFREE(b);
+    b = NULL;
 
     modePrev = mode;
+    if( !rc ) firstIter = 0;
+    if( rc == -1 && globDoublePrecision && !noScale ){
+        rc = -2;
+    }
 
 	return rc;
 }
 
 ITG pastix_solve(double *x, ITG *neq,ITG *symmetryflag,ITG *nrhs){
     mode = BASIC;
+    usage = MULTI_SOLVE;
     if( modePrev != mode ) pastix_set_globals(mode);	
     ITG rc = pastix_solve_generic(x,neq,symmetryflag,nrhs);
     return rc;
@@ -729,6 +898,7 @@ ITG pastix_solve(double *x, ITG *neq,ITG *symmetryflag,ITG *nrhs){
 
 ITG pastix_solve_as(double *x, ITG *neq,ITG *symmetryflag,ITG *nrhs){
     mode = AS;
+    usage = MULTI_SOLVE;
     if( modePrev != mode ) pastix_set_globals(mode);	
     ITG rc = pastix_solve_generic(x,neq,symmetryflag,nrhs);
     return rc;
@@ -737,13 +907,19 @@ ITG pastix_solve_as(double *x, ITG *neq,ITG *symmetryflag,ITG *nrhs){
 // Invokes pastixFinalize and spmExit which frees everything but the dense LU array and parsec pointer
 void pastix_cleanup(ITG *neq,ITG *symmetryflag){
 	if( redo && !firstIter ){
+        if(spm->values == aupastix) spm->values = NULL;
+        if(spm->values == spm->valuesGPU) spm->valuesGPU = NULL;
+        if(spm->colptr == icolpastix) spm->colptr = NULL;
+        if(spm->rowptr == irowpastix) spm->rowptr = NULL;
 		spmExit( spm );
-		if(spm != NULL) free( spm );
+		if(spm != NULL){
+            free( spm );
+            spm = NULL;
+        }
 			
 		pastixFinalize( &pastix_data );
 	}
 	
-	firstIter = 0;
 	return;
 }
 
@@ -759,34 +935,45 @@ void pastix_main_generic(double *ad, double *au, double *adb, double *aub,
 	 ITG *jq, ITG *nzs3,ITG *nrhs){
 		 
 		 
-	if(*neq==0) return;
+	if(*neq==0){
+        return;
+    }
+    else if(*neq==1){
+        noScale=1;
+    }
 
     pastix_set_globals( mode );	
 	
 	const char* pastix_gpu = getenv("PASTIX_GPU");
 	if(pastix_gpu)
 		gpu = (*pastix_gpu == '1') ? 1 : 0;
+
+    usage = SINGLE_SOLVE;
 	
 	// check mixed precision environment variable
     const char* pastix_mixed = getenv("PASTIX_MIXED_PRECISION");	
-    if( pastix_mixed && globDoublePrecision == 0 ){
+    if( pastix_mixed != NULL ){
         mixed = (*pastix_mixed == '1') ? 1 : 0;
-        globDoublePrecision = (mixed == 0) ? 1 : 0;
     }
     else{
         mixed = 1;
     }
 
-	if(!stickToDouble && mixed == '1'){
-		if(globDoublePrecision == 1)
+	if( stickToDouble == 0 && mixed == 1 ){
+		if( globDoublePrecision == 1 ){
 			forceRedo = 1;
-
+        }
 		globDoublePrecision = 0;
 	}
+    else{
+		globDoublePrecision = 1;
+    }
 
-
-    if(*inputformat==3){
-        forceRedo=1;
+    // use double precision for inputformat 3 like mortar (better perfromance and convergence)
+    if( pastix_mixed == NULL && *inputformat == 3 ){
+        globDoublePrecision = 1;
+        forceRedo = 0;
+        stickToDouble = 1;
     }
 
 	// backup b in case mixed precision solve corrupts the original array
@@ -819,6 +1006,44 @@ void pastix_main_generic(double *ad, double *au, double *adb, double *aub,
 	clock_gettime(CLOCK_MONOTONIC, &stepCleanUpEnd); 
 	clock_gettime(CLOCK_MONOTONIC, &stepInitStart); 
 	
+    // scale the matrix with diagonals to 1
+    if( *inputformat !=3 && !noScale ){
+        ITG i=0;
+
+	    #pragma omp parallel for 
+        for(i=0;i<*neq;i++){
+            b[i] /= ad[i];
+        }
+
+        double normb=0;
+        #pragma omp parallel for reduction(+:normb)
+        for(i=0;i<*neq;i++){
+            normb += pow(b[i],2);
+        }
+        normb = sqrt(normb);
+
+        if( normb < 1e-9 ){
+            printf("||b|| getting too small with scaling, boost it statically\n");
+            double scal = 1e-6/normb;
+		    //memcpy(b, b_backup, sizeof(double) * (*nrhs)*(*neq));
+	        #pragma omp parallel for 
+            for(i=0;i<*neq;i++){
+                b[i] *= scal;
+            }
+
+    	    #pragma omp parallel for 
+            for(i=0;i<icolpastix[*neq]-1;i++){
+                aupastix[i] *= scal/ad[irowpastix[i]-1];
+            }
+        }
+        else{
+    	    #pragma omp parallel for 
+            for(i=0;i<icolpastix[*neq]-1;i++){
+                aupastix[i] /= ad[irowpastix[i]-1];
+            }
+        }
+    }
+
 	//invoke init
 	pastix_init(ad,au,adb,aub,sigma,icol,irow, 
 		 neq,nzs,symmetryflag,inputformat,jq,nzs3);
@@ -834,26 +1059,32 @@ void pastix_main_generic(double *ad, double *au, double *adb, double *aub,
 	clock_gettime(CLOCK_MONOTONIC, &stepSolveStart); 
 	
 	// if solve does not converge
-	if(pastix_solve_generic(b,neq,symmetryflag,nrhs) == -1){
+    ITG rc = pastix_solve_generic(b,neq,symmetryflag,nrhs);
+	if( rc == -1){
 		
 		// Give up, if we tried it with double precision, use backup b otherwise
 		if(globDoublePrecision == 1){
             printf("PaStiX could not converge to a valid result\n");
-			exit(1);
-		}
+            exit(5);
+        }
         else{
 		    memcpy(b, b_backup, sizeof(double) * (*nrhs)*(*neq));
+            printf("falling back to double precision\n");
+            globDoublePrecision = 1;
+            forceRedo = 1;
+    	    stickToDouble = 1;
+    	    mixedFailed++;
+        	
+        	// call pastix_main recursively, but now in double precision
+        	pastix_main_generic(ad, au, adb, aub, sigma, b, icol, irow, neq, nzs, symmetryflag, inputformat, jq, nzs3, nrhs);
         }
         
         // make sure that we switch to double and do not reuse in the next iteration
-        globDoublePrecision = 1;
-		forceRedo = 1;
-		stickToDouble = 1;
-		mixedFailed++;
-		
-	 	// call pastix_main recursively, but now in double precision
-		pastix_main_generic(ad, au, adb, aub, sigma, b, icol, irow, neq, nzs, symmetryflag, inputformat, jq, nzs3, nrhs);
-		
+        dparm[DPARM_EPSILON_REFINEMENT] 	= 1e-12;
+        dparm[DPARM_EPSILON_MAGN_CTRL]  	= .0;
+        iparm[IPARM_ITERMAX]            	= 70;
+        iparm[IPARM_GMRES_IM]            	= 70;
+
 		// if we do not converge with mixed precision for the third time, permanently switch to double precision
 		if(mixedFailed <= 2){
 			stickToDouble = 0;
@@ -862,6 +1093,15 @@ void pastix_main_generic(double *ad, double *au, double *adb, double *aub,
 			
 		return;
 	}
+    else if( rc == -2){
+	    memcpy(b, b_backup, sizeof(double) * (*nrhs)*(*neq));
+        printf("turning diagonal scaling off\n");
+        forceRedo = 1;
+        noScale = 1;
+    	
+    	// call pastix_main recursively, but now in double precision
+    	pastix_main_generic(ad, au, adb, aub, sigma, b, icol, irow, neq, nzs, symmetryflag, inputformat, jq, nzs3, nrhs);
+    }
 	else{
 		forceRedo = 0;
 	}
@@ -926,9 +1166,10 @@ void pastix_main_generic(double *ad, double *au, double *adb, double *aub,
 	
 	
     SFREE(b_backup);
+    b_backup = NULL;
+
 	return;
 	
-	
 }
 
 
@@ -938,6 +1179,7 @@ void pastix_factor_main(double *ad, double *au, double *adb, double *aub,
 		ITG *jq, ITG *nzs3){
 
         mode = BASIC;
+        usage = MULTI_SOLVE;
         pastix_factor_main_generic(ad, au, adb, aub, sigma, icol, irow, neq,
                                     nzs, symmetryflag, inputformat, jq, nzs3);
 
@@ -950,6 +1192,7 @@ void pastix_factor_main_as(double *ad, double *au, double *adb, double *aub,
 		ITG *jq, ITG *nzs3){
 
         mode = AS;
+        usage = MULTI_SOLVE;
         pastix_factor_main_generic(ad, au, adb, aub, sigma, icol, irow, neq,
                                     nzs, symmetryflag, inputformat, jq, nzs3);
 
@@ -1012,7 +1255,8 @@ void pastix_set_globals(char mode){
         temp2->jqPrev = jqPrev;
         temp2->inputformatPrev = inputformatPrev;
         temp2->basePrev = basePrev;
-        
+        temp2->offsetPrev = offsetPrev;
+
         // Current sparse matrix in CSC
         temp2->aupastix = aupastix;
         temp2->icolpastix = icolpastix;
@@ -1029,7 +1273,7 @@ void pastix_set_globals(char mode){
         
         // GPU active or not
         temp2->gpu = gpu;
-
+        
         // Store how many nzs the merged Matrix has
         temp2->nzsTotal = nzsTotal;
         
@@ -1076,6 +1320,7 @@ void pastix_set_globals(char mode){
         jqPrev = temp->jqPrev;
         inputformatPrev = temp->inputformatPrev;
         basePrev = temp->basePrev;
+        offsetPrev = temp->offsetPrev;
         
         // Current sparse matrix in CSC
         aupastix = temp->aupastix;
