@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2020 Guido Dhondt
+!              Copyright (C) 1998-2021 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -36,7 +36,8 @@
      &  ttime,qaold,cs,mcs,output,physcon,ctrl,typeboun,iline,ipol,inl,
      &  ipoinp,inp,fmpc,tieset,ntie,tietol,ipoinpc,nslavs,t0g,t1g,nprop,
      &  ielprop,prop,mortar,nintpoint,ifacecount,islavsurf,pslavsurf,
-     &  clearini,ier,vel,nef,velo,veloo,ne2boun)
+     &  clearini,ier,vel,nef,velo,veloo,ne2boun,heading,network,
+     &  irestartread,nfc,ndc,coeffc,ikdc,edc)
 !
       implicit none
 !
@@ -45,6 +46,7 @@
       character*6 prlab(*)
       character*8 lakon(*)
       character*20 labmpc(*),sideload(*)
+      character*66 heading(*)
       character*80 orname(*),amname(*),matname(*)
       character*81 set(*),prset(*),tieset(3,*),cbody(*)
       character*87 filab(*)
@@ -62,9 +64,9 @@
      &  inotr(*),nprop,ielprop(*),mortar,nintpoint,ifacecount,
      &  namta(*),iamt1(*),ielmat(*),nodebounold(*),ndirbounold(*),
      &  iponor(*),knor(*),iponoel(*),inoel(*),rig(*),islavsurf(*),
-     &  nshcon(*),ncocon(*),ics(*),infree(*),ier,
+     &  nshcon(*),ncocon(*),ics(*),infree(*),ier,network,
      &  nener,irestartstep,irestartread,irstrt(*),istat,n,i,key,
-     &  iprestr,mcs,maxlenmpc,iline,ipol,inl,
+     &  iprestr,mcs,maxlenmpc,iline,ipol,inl,nfc,ndc,ikdc(*),
      &  ipoinp(2,*),inp(3,*),ntie,ibody(*),nbody,nslavs,nef,
      &  ne2boun(2,*)
 !
@@ -73,16 +75,15 @@
      &  trab(*),amta(*),t0(*),t1(*),prestr(*),veold(*),tietol(*),
      &  vold(*),xbounold(*),xforcold(*),xloadold(*),t1old(*),eme(*),
      &  xnor(*),thicke(*),offset(*),t0g(*),t1g(*),clearini(*),
-     &  shcon(*),cocon(*),sti(*),ener(*),xstate(*),prop(*),
-     &  ttime,qaold(2),cs(17,*),physcon(*),pslavsurf(*),
+     &  shcon(*),cocon(*),sti(*),ener(*),xstate(*),prop(*),edc(*),
+     &  ttime,qaold(2),cs(17,*),physcon(*),pslavsurf(*),coeffc(*),
      &  ctrl(*),fmpc(*),xbody(*),xbodyold(*),vel(*),velo(*),veloo(*)
 !
-      irestartread=0
       irestartstep=0
 !
       do i=2,n
          if(textpart(i)(1:4).eq.'READ') then
-            irestartread=1
+            irestartread=2
          elseif(textpart(i)(1:5).eq.'STEP=') then
             read(textpart(i)(6:15),'(i10)',iostat=istat) irestartstep
             if(istat.gt.0) then
@@ -111,7 +112,8 @@
          endif
       enddo
 !
-      if(irestartread.eq.1) then
+      if(irestartread.eq.2) then
+        irestartread=1
         call restartread(istep,nset,nload,nforc, nboun,nk,ne,
      &  nmpc,nalset,nmat,ntmat_,npmat_,norien,nam,nprint,mi,
      &  ntrans,ncs_,namtot,ncmat_,mpcfree,maxlenmpc,
@@ -132,7 +134,7 @@
      &  output,physcon,ctrl,typeboun,fmpc,tieset,ntie,tietol,nslavs,
      &  t0g,t1g,nprop,ielprop,prop,mortar,nintpoint,ifacecount,
      &  islavsurf,pslavsurf,clearini,irstrt,vel,nef,velo,veloo,
-     &  ne2boun)
+     &  ne2boun,heading,network,nfc,ndc,coeffc,ikdc,edc)
       endif
 !
       call getnewline(inpc,textpart,istat,n,key,iline,ipol,inl,
