@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2020 Guido Dhondt                          */
+/*              Copyright (C) 1998-2021 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -23,15 +23,15 @@
 #include "CalculiX.h"
 
 void resultsini(ITG *nk,double *v,ITG *ithermal,char *filab,ITG *iperturb,
-		 double *f,double *fn,ITG *nactdof,ITG *iout,double *qa,
-		 double *vold,double *b,ITG *nodeboun,ITG *ndirboun,
-		 double *xboun,ITG *nboun,ITG *ipompc,ITG *nodempc,
-		 double *coefmpc,char *labmpc,ITG *nmpc,ITG *nmethod,
-		 double *cam,ITG *neq,double *veold,double *accold,
-		 double *bet,double *gam,double *dtime,ITG *mi,double *vini,
-		 ITG *nprint,char *prlab,ITG *intpointvarm,ITG *calcul_fn,
-		 ITG *calcul_f,ITG *calcul_qa,ITG *calcul_cauchy,ITG *ikin,
-		 ITG *intpointvart,char *typeboun,ITG *num_cpus){
+		double *f,double *fn,ITG *nactdof,ITG *iout,double *qa,
+		double *vold,double *b,ITG *nodeboun,ITG *ndirboun,
+		double *xboun,ITG *nboun,ITG *ipompc,ITG *nodempc,
+		double *coefmpc,char *labmpc,ITG *nmpc,ITG *nmethod,
+		double *cam,ITG *neq,double *veold,double *accold,
+		double *bet,double *gam,double *dtime,ITG *mi,double *vini,
+		ITG *nprint,char *prlab,ITG *intpointvarm,ITG *calcul_fn,
+		ITG *calcul_f,ITG *calcul_qa,ITG *calcul_cauchy,ITG *ikin,
+		ITG *intpointvart,char *typeboun,ITG *num_cpus,ITG *mortar){
 
     ITG mt,i,j,node,ndir,ist,index,incrementalmpc;
 
@@ -53,6 +53,8 @@ void resultsini(ITG *nk,double *v,ITG *ithermal,char *filab,ITG *iperturb,
 			    continue;
 			}
 			v[mt*i+j]+=bnac;
+			/* FORTRAN(addshell,(nactdof,&i,b,mi,iperturb,
+					     nmethod,cam,v));*/
 			if((iperturb[0]!=0)&&(abs(*nmethod)==1)){
 			    if(fabs(bnac)>cam[0]){
 				cam[0]=fabs(bnac);
@@ -88,26 +90,7 @@ void resultsini(ITG *nk,double *v,ITG *ithermal,char *filab,ITG *iperturb,
 
 	    if(ithermal[0]!=2){
 		iniparll(&mt,nactdof,b,v,veold,accold,bet,gam,
-			 dtime,cam,nk,num_cpus);
-/*		scal1=*bet**dtime**dtime;
-		scal2=*gam**dtime;
-
-		for(i=0;i<*nk;i++){
-		    for(j=1;j<mt;j++){
-			if(nactdof[mt*i+j]>0){
-			    bnac=b[nactdof[mt*i+j]-1];
-			}else{
-			    continue;
-			}
-			v[mt*i+j]+=scal1*bnac;
-			if(fabs(scal1*bnac)>cam[0]){
-			    cam[0]=fabs(scal1*bnac);
-			    cam[3]=nactdof[mt*i+j]-0.5;
-			}
-			veold[mt*i+j]+=scal2*bnac;
-			accold[mt*i+j]+=bnac;
-		    }
-		    }*/
+			 dtime,cam,nk,num_cpus,mortar);
 	    }
 	    
 	    /* transient thermal step */
@@ -388,7 +371,7 @@ void resultsini(ITG *nk,double *v,ITG *ithermal,char *filab,ITG *iperturb,
 
 /*!
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2020 Guido Dhondt
+!              Copyright (C) 1998-2021 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
