@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2020 Guido Dhondt
+!     Copyright (C) 1998-2021 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -47,7 +47,7 @@
 !
 !     initial guess
 !
-      Qred=abs(xflow)*dsqrt(Ts)/(A*pt)
+      Qred=dabs(xflow)*dsqrt(Ts)/(A*pt)
       Tt=Ts*(1+(Qred**2/C**2))
 !     
 !     adiabatic
